/*
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package examples.rmisocfac;

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;

public class HelloImpl
    extends UnicastRemoteObject
    implements Hello
{
    /* Name of remote object in registry */
    private String name;
  
    /* 
     * Constructor calls constructor of superclass with
     * SocketType parameter.
     */
    public HelloImpl(String s) throws RemoteException {
        super(0, new SocketType("xor", null, null));
	name = s;
    }

    /* 
     * Remote method returns String "Hello World!"
     * when invoked.
     */
    public String sayHello() throws RemoteException {
        return  "Hello World!";
    }
  
    public static void main(String args[]) {
      
        // create and install a socket factory
        try {
	    RMISocketFactory.setSocketFactory(new MultiRMISocketFactory());
	} catch (IOException e) {
	    System.err.println("HelloImpl: setSocketFactory failed");
	    e.printStackTrace();
	}

	try {
	    HelloImpl obj = new HelloImpl("HelloServer");
	    Naming.rebind("/HelloServer", obj);
	    System.out.println("HelloServer bound in registry");
	} catch (Exception e) {
	    System.out.println("HelloImpl err: " + e.getMessage());
	    e.printStackTrace();
	}
    }
} 
