/*
 *  File:  MyPolygon.java
 *
 *  Custom polygon class for Java 1.0.2
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.awt.Polygon;
import java.awt.Graphics;
import java.awt.Point;

class MyPolygon extends Polygon {

   // MyPolygon constructor #1:
   public MyPolygon() {
      // invoke the no-argument constructor of the superclass:
      super();
   }

   // MyPolygon constructor #2:
   public MyPolygon( int[] xpoints, int[] ypoints, int npoints ) {
       
      // invoke a constructor of the superclass:
      super( xpoints, ypoints, npoints );
      
   }

   /*
    *  MyPolygon constructor #3
    *
    *  A regular n-gon with center (x,y) and radius r
    *
    */
    
   public MyPolygon( int x, int y, int r, int n ) {
       
      // invoke the no-argument constructor of the superclass:
      super();
      
      double theta = 2 * Math.PI/n;
      double cos_theta = Math.cos( theta );
      double sin_theta = Math.sin( theta );
      
      int dx, dy;
      int old_x, old_y;
    
      int new_x = x, new_y = y - r;
      this.addPoint( new_x, new_y );
      for ( int i = 1; i < n; i++ ) {
         old_x = new_x; old_y = new_y;
         dx = old_x - x; dy = old_y - y;
         new_x = ( int ) Math.round( x + dx * cos_theta - dy * sin_theta );
         new_y = ( int ) Math.round( y + dx * sin_theta + dy * cos_theta );
         this.addPoint( new_x, new_y );
      }
      
   }

   // instance methods:
   public void draw( Graphics g ) {
      g.drawPolygon( this );
   }
   public void fill( Graphics g ) {
      g.fillPolygon( this );
   }
   
   // rotate this polygon:
   public MyPolygon rotate( double theta ) {
   
      int dx, dy;
      final double cos_theta = Math.cos( theta );
      final double sin_theta = Math.sin( theta );
      
      int x = this.getBoundingBox().x;
      int y = this.getBoundingBox().y;
      for ( int i = 0; i < this.npoints; i++ ) {
         dx = this.xpoints[i] - x;
         dy = this.ypoints[i] - y;
         this.xpoints[i] = 
          ( int ) Math.round( x + dx * cos_theta - dy * sin_theta );
         this.ypoints[i] = 
          ( int ) Math.round( y + dx * sin_theta + dy * cos_theta );
      }
      
	  // return the rotated polygon as a side effect:
      return this;
      
   }
   
   // overload Polygon.addPoint( int, int ):
   public void addPoint( Point p ) {
      super.addPoint( p.x, p.y );
   }
   
}
