/*  Knight's tour applet with double buffering	*/
/*  Ernest Sibert, July 1996			*/

import java.awt.*;
import java.io.*;
import java.net.*;


public class Knight extends java.applet.Applet implements Runnable
 { 
   public static final Font cfont = new Font("Helvetica", Font.BOLD, 18);

   static Image knight;		/* The knight icon (32x32) */

   int bsize =  8,	/* Board dimension (squares)	*/
       sqd   = 48,	/* Square dimension (pixels)	*/
       ktd   = 32,	/* Knight dimension (pixels)	*/
       ktin  =  8,	/* inset for Knight (pixels)	*/
       nxin,		/* x offset for numbers		*/
       nyin;		/* y offset for numbers		*/

   String tFile;	/* Name of file with tour	*/

   Thread kThread;		/* Runs the Knight	*/

   Color bg = Color.blue,	/* Standard background	*/
	 fg = Color.white,	/* Standard foreground	*/
	 sr = Color.red,	/* Red square		*/
	 sb = Color.black;	/* Black square		*/

   int   kh = 0, kv = 0,	/* Knight position (squares)	*/
                 ki = 0;	/* Number of moves so far	*/

   int  tourLength,		/* Number of squares in the tour */
	tourk     = 0;		/* Position in the tour		 */
   Point tour[];		/* The tour, defined on		 */
				/*  [0..tourLength] (inclusive)  */

   String board[][];		/* The chess board	*/

	/* Board coding:  null = not visited		*/
	/*		  " 0" = Start of tour		*/
	/*                " 3" = Position in tour	*/
        /*		  "12" = Position in tour	*/

   Image  iBuff;		/* Image buffer			*/
   Graphics gcx;		/* Graphics context for iBuff	*/
   boolean ready = false;	/* To synchronize startup	*/


   /* Initialize the applet.  The compilcations regarding the knight	*/
   /* icon result from the fact that getImage(...) returns without	*/
   /* actually loading the image, so that early drawings may use an	*/
   /* incomplete, even non-existant icon.  The method `prepareImage',	*/
   /* initiates loading and returns true if the image is complete and	*/
   /* ready to draw.							*/

   public void init()
    { String bgs, fgs;				/* For color parameters */
      knight = getImage(getCodeBase(), "Knight.gif");	/* Specify image */
      boolean kready = prepareImage(knight, this);	/* Start loading */
      board = new String[bsize][bsize];
      tFile = getParameter("tourfile");
      setFont(cfont);  setBackground(bg);  setForeground(fg);
      getAppletContext().showStatus("Loading the tour");
      board = new String[bsize][bsize];
      loadTour();
      FontMetrics fm = getFontMetrics(cfont);
      int sh = fm.getAscent();  int sw = fm.stringWidth("77");
      nxin = (sqd - sw)/2;  nyin = (sqd + sh)/2;
      iBuff = createImage(size().width, size().height);
      gcx = iBuff.getGraphics();
      gcx.setFont(cfont);
      getAppletContext().showStatus("Loading image from server....");
      while ( ! prepareImage(knight, this) ) pause(200);  /* Wait for icon */
      getAppletContext().showStatus("Ready");
      ready = true;			/* Initialization is finished */
    }


   /* Load the tour from file tFile.  The file has a sequence of int's	*/
   /* of the form  <tour_length> <x> <y> ... <x> <y>  giving the	*/
   /* sequence of knight positions.					*/
   /* Uses StreamTokenizer assuming file has nothing but numbers and	*/
   /* comments identified by `//' (to end of line).  Of course, there	*/
   /* can be anything at all after the data that's read.		*/

   void loadTour()
    { try
       { StreamTokenizer data =
	   new StreamTokenizer(new BufferedInputStream(
			new URL(getCodeBase(), tFile).openStream()));
	 data.parseNumbers();  data.slashSlashComments(true);
	 int tok = data.nextToken();  tourLength = (int) data.nval;
	// System.out.println("tourLength = " + tourLength); /* Diag. */
	 if ( tourLength >= 0 )
	  { tour = new Point[tourLength + 1];
	    for ( int i = 0; i <= tourLength;  i++ )
	     { int x, y;
	       tok = data.nextToken();  x = (int) data.nval;
	       tok = data.nextToken();  y = (int) data.nval;
	      // System.out.println("Point: " + x + ", " + y); /* Diag. */
	       tour[i] = new Point(x, y);
	     }
	  }
       }
      catch (IOException ex)
       { System.out.println("Couldn't read file for tour - "
			      + ex.getMessage());
       }
    }

   /* Delay for specified time (in msec.) */

   void pause(int delay)
    { try { Thread.sleep(delay); }
      catch (InterruptedException ex) { }
    }

   /* Start process - The wait until ready and call to drawBoard have	*/
   /* been moved here, since Netscape reported a security violation	*/
   /* when these were at the beginning or `run'.  Appletviewer was	*/
   /* happy with either version.					*/

   public void start()
    { while ( !ready )	pause(500);		/* Wait until ready	*/
      getAppletContext().showStatus("Drawing board");
      drawBoard();
      if ( kThread == null )
       { kThread = new Thread(this);
	 kThread.start(); }
    }

   /* Stop process  */

   public void stop()
    { if ( kThread != null )
       synchronized (iBuff)
	{ kThread.stop();  kThread = null; }
    }


   /* The tour procedure, runs in kThread.  The initial busy wait	*/
   /* assures that start-up activity is completed.			*/

   public void run()
    { int delay = 1200;				/* Time between moves	*/
      repaint();				/* Paint initial state	*/
      getAppletContext().showStatus("Off and running");
      while ( ki < tourLength)			/* runs in kThread	*/
       { /* Record current position: */
	 String seq = String.valueOf(ki);
	 if ( 0 <= ki && ki < 10 ) seq = " " + seq;
	 synchronized (iBuff)			/* Update state		*/
	  { board[kh][kv] = seq;
	    ki++;  Point next = tour[ki];	/* Next square		*/
	    kh = next.x;  kv = next.y;		/* New board coords	*/
	  }
	 pause(delay);
	 moveKnight();  repaint();		/* Move and Repaint 	*/
       }
    }


   /* Move the knight to the next square in the buffer.  Must be */
   /* called with ki >= 1 (knight has moved).			 */

   void moveKnight()
    { Point z;  int i, j, xloc, yloc;		/* For square coordinates */
      z = tour[ki - 1];  i = z.x;  j = z.y;	/* Cell left by knight	  */
      xloc = sqd*i;  yloc = sqd*j;
      String ks = board[i][j];
      synchronized (iBuff)
       { gcx.setColor( ((i+j)&1) == 0 ? sr : sb );
	 gcx.fillRect(xloc, yloc, sqd, sqd);
	 gcx.setColor(fg);
	 gcx.drawString(ks, xloc+nxin, yloc+nyin);	/* Sequence number */
	 gcx.drawImage(knight, sqd*kh + ktin, sqd*kv + ktin, this);  /* Kt */
       }
     }


   public void update(Graphics g)		/* Flicker reduction	*/
    { paint(g); }


   public void repaint()
    { paint(getGraphics()); }


   public void paint(Graphics g)		/* Using buffer iBuff	*/
    { if ( ready )
        synchronized (iBuff) {g.drawImage(iBuff, 0, 0, this);}
    }


   /* Draw board in buffer from scratch using current knight position */

   void drawBoard()
    { synchronized (iBuff)
       { for ( int i = 0;  i < bsize;  i++ )
	  for ( int j = 0;  j < bsize;  j++ )
	   { int xloc = sqd*i,  yloc = sqd*j; 
	     gcx.setColor( ((i+j)&1) == 0 ? sr : sb );
	     gcx.fillRect(xloc, yloc, sqd, sqd);
             String ks = board[i][j];
	     if ( ks != null )			/* Draw tour sequence	*/
	      { gcx.setColor(fg);
	        gcx.drawString(ks, xloc+nxin, yloc+nyin);
	      }

            }
         gcx.drawImage(knight, sqd*kh + ktin, sqd*kv + ktin, this);
       }
    }

   public void destroy()
    { gcx.dispose();  super.destroy(); }

 }
