import java.awt.*;
import java.awt.event.*;

/**
* <h4>ChooserPanel Bean/Widget</h4>
*
* <img src="ChooserPanel.jpg" alt="screen shot">
* 
* <p>A Java Bean/Widget with two lists - a choice list and
* a selected list, which enables the user to add/remove
* selections from the given list.
*
* <p>Feel free to re-use any part of this code.
*
* <p>Jamie Hall<br>
* <a href="mailto:jamihall@ulster.net">jamihall@ulster.net</a><br>
* <a href="http://www.ulster.net/~jamihall">http://www.ulster.net/~jamihall</a>
*
* <p>Download a <a href="chooser.jar">JAR file</a> with all the
* compiled classes, or view <a href="ChooserPanel.java">the source.</a>
*
* @author Jamie Hall
* @version 10/08/97
*/
public class ChooserPanel extends Panel {
	List choiceList,chosenList;
	Button removeBtn,removeAllBtn,addBtn,addAllBtn;
	boolean multipleMode = false,removeWhenChosen = false;

   /**
	* Constructor.
	*/
	public ChooserPanel() {
		setLayout(new BorderLayout(5,5));

		Panel p = new Panel();
		p.setLayout(new GridBagLayout());
		add("Center",p);

		// Choice List.
		choiceList = new List();
		constrain(p,choiceList,0,0,1,6,GridBagConstraints.BOTH,
			GridBagConstraints.CENTER,100,100,0,0,0,0);

		// Chosen list.
		chosenList = new List();
		constrain(p,chosenList,2,0,1,6,GridBagConstraints.BOTH,
			GridBagConstraints.CENTER,100,100,0,0,0,0);

		// Add Button.
		addBtn = new Button("Add ->");
		constrain(p,addBtn,1,1,1,1,GridBagConstraints.HORIZONTAL,
			GridBagConstraints.CENTER,10,1,0,15,0,15);
		addBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int[] indexes  = getChoiceSelectedIndexes();
				if (indexes != null) {
					for (int i = 0; i < indexes.length; i++) {
						addChosen(getChoiceItem(indexes[i]));
						if (removeWhenChosen)
							removeChoice(indexes[i]);
					}
				}
			}
		});

		// Add all Button.
		addAllBtn = new Button("Add All ->");
		constrain(p,addAllBtn,1,2,1,1,GridBagConstraints.HORIZONTAL,
			GridBagConstraints.CENTER,10,1,5,15,0,15);
		addAllBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String[] s = getChoiceItems();
				if (s != null) {
					for (int i = 0; i < s.length; i++)
						addChosen(s[i]);
				}
			}
		});

		// Remove button.
		removeBtn = new Button("<- Remove");
		constrain(p,removeBtn,1,3,1,1,GridBagConstraints.HORIZONTAL,
			GridBagConstraints.CENTER,10,1,5,15,0,15);
		removeBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int[] indexes = getChosenSelectedIndexes();
				if (indexes != null) {
					for (int i = 0; i < indexes.length; i++) {
						int j = indexes[i];
						if (i > 0)
							--j;
						removeChosen(j);
					}
				}
			}
		});

		// Remove all button.
		removeAllBtn = new Button("<- Remove All");
		constrain(p,removeAllBtn,1,4,1,1,GridBagConstraints.HORIZONTAL,
			GridBagConstraints.CENTER,10,1,5,15,0,15);
		removeAllBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeAllChosens();
			}
		});
	}

   /**
	* Adds the specified item to the end of the
	* choice list.
	*/
	public void addChoice(String s) {
		choiceList.add(s);
	}

   /**
	* Adds the specified item at the specified
	* index to the choice list.
	*/
	public void addChoice(String s, int i) {
		choiceList.add(s,i);
	}

   /**
	* Deselects the item at the specified index
	* in the choice list.
	*/
	public void deselectChoice(int i) {
		choiceList.deselect(i);
	}

   /**
	* Gets the item associated with the specified index
	* in the choice list.
	*/
	public String getChoiceItem(int i) {
		return choiceList.getItem(i);
	}

   /**
	* Gets the number of items in the choice list.
	*/
	public int getChoiceItemCount() {
		return choiceList.getItemCount();
	}

   /**
	* Gets the items in the choice list.
	*/
	public String[] getChoiceItems() {
		return choiceList.getItems();
	}
    
   /**
	* Get the number of visible lines in the choice list.
	*/
	public int getChoiceRows() {
		return choiceList.getRows();
	}

   /**
	* Gets the index of the selected item on the choice list.
	*/
	public int getChoiceSelectedIndex() {
		return choiceList.getSelectedIndex();
	}
	
   /**
	* Gets the selected indexes on the choice list.
	*/
	public int[] getChoiceSelectedIndexes() {
		return choiceList.getSelectedIndexes();
	}

   /**
	* Get the selected item on the choice list.
	*/
	public String getChoiceSelectedItem() {
		return choiceList.getSelectedItem();
	}

   /**
	* Get the selected items on the choice list.
	*/
	public String[] getChoiceSelectedItems() {
		return choiceList.getSelectedItems();
	}

   /**
	* Returns the selected items on the choice list in
	* an array of Objects.
	*/
	public Object[] getChoiceSelectedObjects() {
		return choiceList.getSelectedObjects();
	}

   /**
	* Gets the index of the item that was last made
	* visible by the method makeChoiceVisible in the
	* choice list.
	*/
	public int getChoiceVisibleIndex() {
		return choiceList.getVisibleIndex();
	}

   /**
	* Determines if the specified item in the
	* choice list is selected.
	*/
	public boolean isChoiceIndexSelected(int i) {
		return choiceList.isIndexSelected(i);
	}
	
   /**
	* Makes the item at the specified index visible
	* in the choice list.
	*/
	public void makeChoiceVisible(int i) {
		choiceList.makeVisible(i);
	}

   /**
	* Remove the item at the specified position
	* from the choice list.
	*/
	public void removeChoice(int i) {
		choiceList.remove(i);
	}

   /**
	* Removes the first occurrence of an item
	* from the choice list.
	*/
	public void removeChoice(String s) {
		choiceList.remove(s);
	}

   /**
	* Removes all items from the choice list.
	*/
	public void removeAllChoices() {
		choiceList.removeAll();
	}
   
   /**
	* Replaces the item at the specified index in the
	* choice list with the new string.
	*/
	public void replaceChoiceItem(String s, int i) {
		choiceList.replaceItem(s,i);
	}

   /**
	* Selects the item at the specified index in the
	* choice list.
	*/
	public void selectChoice(int i) {
		choiceList.select(i);
	}

   /**
	* Adds the specified item to the end of the
	* chosen list.
	*/
	public void addChosen(String s) {
		boolean alreadyChosen = false;
		String[] ss = getChosenItems();
		if (ss != null) {
			for (int i = 0; i < ss.length; i++) {
				if (s.equals(ss[i])) {
					alreadyChosen = true;
					break;
				}
			}

			if (!alreadyChosen)
				chosenList.add(s);
		}
	}

   /**
	* Adds the specified item at the specified
	* index to the chosen list.
	*/
	public void addChosen(String s, int i) {
		chosenList.add(s,i);
	}

   /**
	* Deselects the item at the specified index
	* in the chosen list.
	*/
	public void deselectChosen(int i) {
		chosenList.deselect(i);
	}

   /**
	* Gets the item associated with the specified index
	* in the chosen list.
	*/
	public String getChosenItem(int i) {
		return chosenList.getItem(i);
	}

   /**
	* Gets the number of items in the chosen list.
	*/
	public int getChosenItemCount() {
		return chosenList.getItemCount();
	}

   /**
	* Gets the items in the chosen list.
	*/
	public String[] getChosenItems() {
		return chosenList.getItems();
	}
    
   /**
	* Get the number of visible lines in the chosen list.
	*/
	public int getChosenRows() {
		return chosenList.getRows();
	}

   /**
	* Gets the index of the selected item on the chosen list.
	*/
	public int getChosenSelectedIndex() {
		return chosenList.getSelectedIndex();
	}
	
   /**
	* Gets the selected indexes on the chosen list.
	*/
	public int[] getChosenSelectedIndexes() {
		return chosenList.getSelectedIndexes();
	}

   /**
	* Get the selected item on the chosen list.
	*/
	public String getChosenSelectedItem() {
		return chosenList.getSelectedItem();
	}

   /**
	* Get the selected items on the chosen list.
	*/
	public String[] getChosenSelectedItems() {
		return chosenList.getSelectedItems();
	}

   /**
	* Returns the selected items on the chosen list in
	* an array of Objects.
	*/
	public Object[] getChosenSelectedObjects() {
		return chosenList.getSelectedObjects();
	}

   /**
	* Gets the index of the item that was last made
	* visible by the method makeChosenVisible in the
	* chosen list.
	*/
	public int getChosenVisibleIndex() {
		return chosenList.getVisibleIndex();
	}

   /**
	* Determines if the specified item in the
	* chosen list is selected.
	*/
	public boolean isChosenIndexSelected(int i) {
		return chosenList.isIndexSelected(i);
	}
	
   /**
	* Makes the item at the specified index visible
	* in the chosen list.
	*/
	public void makeChosenVisible(int i) {
		chosenList.makeVisible(i);
	}

   /**
	* Remove the item at the specified position
	* from the chosen list.
	*/
	public void removeChosen(int i) {
		chosenList.remove(i);
	}

   /**
	* Removes the first occurrence of an item
	* from the chosen list.
	*/
	public void removeChosen(String s) {
		chosenList.remove(s);
	}

   /**
	* Removes all items from the chosen list.
	*/
	public void removeAllChosens() {
		chosenList.removeAll();
	}
   
   /**
	* Replaces the item at the specified index in the
	* chosen list with the new string.
	*/
	public void replaceChosenItem(String s, int i) {
		chosenList.replaceItem(s,i);
	}

   /**
	* Selects the item at the specified index in the
	* chosen list.
	*/
	public void selectChosen(int i) {
		chosenList.select(i);
	}

   /**
	* Sets the flag that determines whether the choice
	* and chosen lists allow multiple selections.
	*/
	public void setMultipleMode(boolean b) {
		multipleMode = b;
		choiceList.setMultipleMode(multipleMode);
		chosenList.setMultipleMode(multipleMode);
	}

   /**
	* Determines whether the choice and chosen lists
	* allow multiple selections.
	*/
	public boolean isMultipleMode() {
		return multipleMode;
	}

   /**
	* Sets the flag that determines whether to delete the
	* ntry from the choice list when something is chosen.
	*/
	public void setRemoveWhenChosen(boolean b) {
		removeWhenChosen = b;
	}

   /**
	* Determines whether entries are deleted from the
	* choice list when something is chosen.
	*/
	public boolean isRemoveWhenChosen() {
		return removeWhenChosen;
	}

   /**
	* This is the main constrain() method. It has arguments
	* for all constraints.
	*/
    protected void constrain(Container container, Component component,
		int grid_x, int grid_y, int grid_width, int grid_height,
		int fill, int anchor, double weight_x, double weight_y,
		int top, int left, int bottom, int right) {

		java.awt.GridBagConstraints c = new java.awt.GridBagConstraints();
		c.gridx = grid_x; c.gridy = grid_y;
		c.gridwidth = grid_width; c.gridheight = grid_height;
		c.fill = fill; c.anchor = anchor;
		c.weightx = weight_x; c.weighty = weight_y;
		if (top+bottom+left+right > 0)
			c.insets = new java.awt.Insets(top, left, bottom, right);

		((java.awt.GridBagLayout)container.getLayout()).setConstraints(component,c);
		container.add(component);
	}

	public static void main(String[] args) {
		Frame f = new Frame();

		ChooserPanel chooserPanel = new ChooserPanel();
		chooserPanel.setMultipleMode(true);
		chooserPanel.addChoice("one");
		chooserPanel.addChoice("two");
		chooserPanel.addChoice("three");
		chooserPanel.addChoice("four");

		f.add(chooserPanel);
		f.pack();
		f.setVisible(true);
	}

}
