import java.awt.*;

/*
 * Note - Copyright 1997 John Jensen.  All Rights Reserved.  See license.txt
 */

// This extension of Dialog implements modal dialogs with a callback interface
// to work around the "modal dialog problem" on Windows.  The "NoteListener"
// interface is used to return an action code to the parent when "continue" is
// pressed.

class Note extends Dialog
{
	int noteAction;
	NoteListener noteListener;

	// Constructor takes a string and an action.  Note that action == 0 implies
	// that there is only an "ok" button, while action != 0 implies a "Continue" and
	// a "Cancel" button.

	public Note(Frame parent, String s, int action)
	{
		super(parent, "Note", true);

		noteListener = (NoteListener)parent;
		noteAction = action;

		Panel center = new Panel();
		center.add(new Label(s));
		add("Center", center);

		Panel south = new Panel();
		if (action != 0)
		{
			south.add(new Button("Continue"));
			south.add(new Button("Cancel"));
		}
		else
			south.add(new Button("Ok"));
		add("South", south);

		resize(300, 100);
	}

	// if we get a "Continue" button, call the parent's noteListener

	public boolean action(Event evt, Object arg)
	{
		if (evt.target instanceof Button )
		{
			if (arg.equals("Continue"))
				noteListener.noteListen(noteAction);
			dispose();
			return true;
		}
		return false;
	}

	// if the user clicks the "go away" box, handle it

	public boolean handleEvent(Event evt)
	{
		if ((evt.id == Event.WINDOW_DESTROY) && (evt.target == this))
		{
			dispose();
			return true;
		}
		return super.handleEvent(evt);
	}
}
