/*
 * Copyright (c) 1997 John Jensen. All rights reserved.
 *
 * This software is FREE FOR COMMERCIAL AND NON-COMMERCIAL USE,
 * provided the following condition is met.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that any copy or derivative of this software or documentation
 * retaining the name "John Jensen" also retains this condition and the
 * following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * CopyrightVersion 1.0
 */

import java.util.*;
import java.awt.*;

public class mpEDIT
{
	private StringMan	stringMan;
	private Properties	strings;
	private PropMan		propMan;
	private Vector		docs;
	private int			untitled_count;

	private Dimension	screenDim;
	private Dimension	windowSize;
	private int			windowOffset;
	private int			windowCount;
	private Point		lastPlace;

	// constructor puts together UI

	public mpEDIT()
	{
		propMan = new PropMan();
		propMan.readProperties();

		stringMan = new StringMan();
		strings = stringMan.getStrings();

		screenDim = Toolkit.getDefaultToolkit().getScreenSize();

		windowSize = new Dimension(screenDim.width * 2/3,screenDim.height * 2/3);
		windowOffset = screenDim.height / 21;

		docs = new Vector(5,5);
		newDoc();
	}

	public int getUntitled()
	{
		return ++untitled_count;
	}

	public synchronized void newDoc()
	{
		DocMan doc = new DocMan(this,strings,propMan);
		if (doc != null)
		{
			TextFrame textFrame = doc.newFrame(getPlace(windowSize),windowSize);
			doc.setUntitled(++untitled_count);
			textFrame.setVisible(true);
			docs.addElement(doc);
		}
	}

	public synchronized void openDoc(DocMan doc, TextFrame textFrame)
	{
		boolean newdoc;
		String filename = getFileForOpen(textFrame);

		if (filename != null)
		{
			newdoc = doc.isBusy();

			if (newdoc)
			{
				doc = new DocMan(this,strings,propMan);
				textFrame = doc.newFrame(getPlace(windowSize),windowSize);
			}

			doc.openDoc(textFrame,filename);

			if (newdoc)
			{
				textFrame.setVisible(true);
				docs.addElement(doc);
			}
		}
	}

	public synchronized void closeDoc(DocMan doc)
	{
		int i,max;

		max = docs.size();

		for (i = 0; i < max; i++ )
		{
			if (doc == (DocMan)docs.elementAt(i))
			{
				docs.removeElementAt(i);
				max--;
				break;
			}
		}

		if (max == 0)
			System.exit(0);
	}

	public Dimension getWindowSize()
	{
		return windowSize;
	}

	public synchronized Point getPlace(Dimension size)
	{
		Point place = null;

		if (lastPlace == null)
		{
			place = new Point(0,0);
		}
		else
		{
			place = new Point(lastPlace);

			place.y += windowOffset;
			place.x += windowOffset;

			if ((place.x + size.width) > screenDim.width)
				place.x = 0;

			if ((place.y + size.height) > screenDim.height)
				place.y = 0;
		}

		lastPlace = place;

		return place;
	}

	private String getFileForOpen(TextFrame textFrame)
	{
		String prompt;
		String filename;
		String pathname;

		prompt = strings.getProperty("DialogOpen");

		FileDialog d = new FileDialog(textFrame, prompt, FileDialog.LOAD);

		d.setFile("*");
		d.setDirectory(".");
		d.setVisible(true);

		filename = d.getFile(); 

		pathname = d.getDirectory() + filename; 

		d.dispose();

		if (filename != null)
			return pathname;
		else
			return null;
	}

	private String findPattern = null;
	private String replacePattern = null;
	private String replaceReplace = null;

	public void setFindPattern(String pat)
	{
		findPattern = new String(pat);
	}

	public void setReplacePattern(String pat)
	{
		replacePattern = new String(pat);
	}

	public void setReplaceReplace(String pat)
	{
		replaceReplace = new String(pat);
	}

	public String getFindPattern()
	{
		return findPattern;
	}

	public String getReplacePattern()
	{
		return replacePattern;
	}

	public String getReplaceReplace()
	{
		return replaceReplace;
	}

	public static void main(String[] args)
	{
		mpEDIT mpe = new mpEDIT();
	}
}

