
package com.magelang;


import java.awt.Panel; 
import java.awt.Graphics; 
import java.awt.Color; 
import java.awt.Dimension; 
import java.awt.FontMetrics; 
import java.awt.Font; 
import java.awt.Component; 
import java.awt.Polygon; 
import java.util.Vector; 
import java.util.Hashtable; 
import java.awt.Insets; 
import java.awt.event.MouseEvent; 
import java.awt.event.MouseListener; 
import java.awt.CardLayout; 
import java.awt.Container; 
import java.awt.Image; 
import java.awt.Rectangle; 
import com.magelang.*;
import java.awt.Cursor;
import java.util.Enumeration;
/**
 * TabSplitter -- allows a user to select from several interface components
 * by clicking a tab at the top of the panel.
 * In addition, it allows the user to merge any tabs together into a
 * a single tab using a SplitterLayout.
 * <p>Each contained component is represented by a tab at the top of the
 * TabSplitter, much like file folders in a file cabinet.  When a
 * tab is clicked, it becomes the "selected" tab and its associated
 * component will be displayed.
 * <p>There are two types of navigational aids provided with the
 * TabSplitter.  If there are more tabs than can be displayed in the
 * current window, two triangle buttons will appear.  These buttons
 * will scroll the set of tabs left and right.
 * <p>There are also two buttons marked "+" and "-".  These
 * buttons move the user through each tab in succession.
 * <p>To properly set up a tab panel, you need to do two things:
 * <ul>
 *    <li>add components to the TabSplitter, using an "add" method.
 *        <br>The order in which panels are added is the 
 *            order in which their tabs will appear.
 *    <li>set a tabText string array to represent the text that
 *        is displayed on each tab.
 * </ul>
 *
 * <p>The thing that sets this tabbed panel apart is how it reacts
 * to mouse drags on its tabs.  Click on a tab and drag it to
 * any other tab.  The "source" tab will be merged into the
 * "target" tab, combined via a panel with a SplitterLayout.
 * The target tab's text will be modified to list both previous
 * tab texts.  In addition, each sub panel that has been merged will
 * have a Button placed above it with the old tab text for that
 * component.  Clicking on this button will separate out the subpanel
 * on its own tab once again.
 *
 * <p><b>Note:</b> It is extremely important that the user of this
 * tab panel not try to directly use the layoutmanger (via getLayout()
 * and setLayout() ).  These two methods could not be overridden
 * to prevent modification, as many GUI builders expect to use it.
 * If you want to switch between tabs, use the "next()" and "previous()"
 * methods provided by TabSplitter, <i>not</i> those of CardLayout.
 * <p>Use this code at your own risk!  MageLang Institute is not
 * responsible for any damage caused directly or indirctly through
 * use of this code.
 
 *<h2>Known problems
 * <ol>
 * <li>Tab heights are fixed -- should vary based on Font used
 * <li>Merge and split algorithms make some assumptions about the components
 *   if they are splitter panels
 * <li>Cannot currently show an explicit panel.
 *   The problem here is that we don't track the names used by the CardLayout to
 *   represent names, and these names may get merged when tabs are merged
 * </ol?
 
 * <p><p>
 * <b>SOFTWARE RIGHTS</b>
 * <p>
 * MageLang support classes, version 1.0, MageLang Institute
 * <p>
 * We reserve no legal rights to this code--it is fully in the
 * public domain. An individual or company may do whatever
 * they wish with source code distributed with it, including
 * including the incorporation of it into commerical software.
 *
 * <p>However, this code cannot be sold as a standalone product.
 * <p>
 * We encourage users to develop software with this code. However,
 * we do ask that credit is given to us for developing it
 * By "credit", we mean that if you use these components or
 * incorporate any source code into one of your programs
 * (commercial product, research project, or otherwise) that
 * you acknowledge this fact somewhere in the documentation,
 * research report, etc... If you like these components and have
 * developed a nice tool with the output, please mention that
 * you developed it using these components. In addition, we ask that
 * the headers remain intact in our source code. As long as these
 * guidelines are kept, we expect to continue enhancing this
 * system and expect to make other tools available as they are
 * completed.
 * <p>
 * The MageLang Support Classes Gang:
 * @version MageLang Support Classes 1.0, MageLang Insitute, 1997
 * @author <a href="http:www.scruz.net/~thetick">Scott Stanchfield</a>, <a href=http://www.MageLang.com>MageLang Institute</a>
 * @see SplitterLayout
 */
public class TabSplitter extends java.awt.Panel implements java.awt.event.MouseListener {
	// Define cursors that we'll use when there we are dragging the tabs
	private static Cursor DEF_CURSOR = new Cursor(Cursor.DEFAULT_CURSOR);
	private static Cursor HAND_CURSOR = new Cursor(Cursor.HAND_CURSOR);
	
	protected transient java.awt.event.ActionListener aActionListener = null;
	
	
	private java.awt.Rectangle bothRect;

   // The color that will be used around the edge of the tab panel
	java.awt.Color fieldBorderColor = null;
	
	// The color that will be used as the background of the tabs
	java.awt.Color fieldTabColor = null;
	
  // Keep track of which tab is the first one visible on the screen
	private int firstVisible = 0;
	
	// Keep track of the graphics context into which we will draw the tabs
	private java.awt.Graphics g1;
	
	private int hslop = 4;
	
	// An image to draw into -- for double buffering
	private java.awt.Image image;
	
	// Keep track of the last size of the panel -- we'll need to create a new
	// double-buffer image if the size of the panel changes
	private int lastH = 0;
	private int lastW = 0;
	
	// The polygons that are used to draw the left and right arrows for scrolling the tabs
	private java.awt.Polygon leftArrow;
	private java.awt.Polygon rightArrow;
	
	// Keeps track of the currently-selected tab
	private int selected = 0;
	
	// Keeps track of the polygons that represent the tabs
	private java.util.Vector tabs;
	
	// Do we have more tabs than fit on the panel?
	private boolean tooManyTabs = false;
	private int vslop = 4;
	
	// A vector of the Strings that are used as the text on the tabs
	private java.lang.String[] fieldTabText = null;
	
	// Which tab is currently being dragged -- -1 means no drag in progress
	private int tabBeingDragged=-1;
	


/**
 * Constructor
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
public TabSplitter() {
	super();
	initialize();
}
/**
 * Adds a component to a container
 * This method is overridden to force a repaint() in bean design mode
 * @param comp (java.awt.Component) -- the component to be added
 * @param constraints (java.lang.Object) -- constraints on the component
 * @param index (int) -- at which position will the component be added
 */
protected void addImpl(Component comp, Object constraints, int index) {
	super.addImpl(comp, constraints, index);
	
	// if the bean is being displayed for a designer, force a repaint()
	if (java.beans.Beans.isDesignTime())
		repaint();
	return;
}
/**
 * conn0:  (TabPanel.mouse.mouseReleased(java.awt.event.MouseEvent) --> TabPanel.selectTab(Ljava.awt.event.MouseEvent;)V)
 * @param arg1 java.awt.event.MouseEvent
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private void conn0(java.awt.event.MouseEvent arg1) {
	try {
		// user code begin {1}
		// user code end
		this.selectTab(arg1);
		// user code begin {2}
		// user code end
	} catch (java.lang.Throwable ivjExc) {
		// user code begin {3}
		// user code end
		handleException(ivjExc);
	}
}
/** Draws the tabs at the top of the tab panel
 *  @param g (java.awt.Graphics) -- the graphics context into which tabs are drawn
 */
protected final void drawTabs(Graphics g) {
	// get the current size of the panel
	Dimension dim = getSize();
	FontMetrics fm = g.getFontMetrics();
	
	// create the polygons that will be used to locate the tabs
	setupTabPolygons(g);
	
	// set the appropriate color
	g.setColor(getTabColor());

	// TBD: should try to resize tabs based on font metrics...

   // make the font BOLD
	Font currentFont = g.getFont();
	Font boldFont = new Font(currentFont.getName(), Font.BOLD, currentFont.getSize());
	
	int tabNum; // walker...

	// get a list of all the panels contained in the TabSplitter	
	Component comp[] = getComponents();
	if (comp.length == 0) return; // if none, just get out

	// if the first visible tab is not the first tab,
	//   force us to use the left/right arrows
	tooManyTabs = (firstVisible != 0);
	
	// for each tab that is visible on the screen	
	for(int num=comp.length-1;num>firstVisible-2;num--) {
		if (num == firstVisible-1) {
			if (selected < firstVisible) break;
			tabNum = selected;
			g.setFont(boldFont);
			}
		else if (num == selected)	continue;
		else tabNum = num;

	  // get ready to paint a tab
		g.setColor(getTabColor());
		Polygon p = (Polygon)tabs.elementAt(tabNum-firstVisible);
		g.fillPolygon(p);
		
		//  draw hilites
		int x[] = p.xpoints;
		int y[] = p.ypoints;
		g.drawLine(x[0],y[0],x[11],y[11]);
			
		g.setColor(getTabColor().darker());
		int i;
		for(i=10;i>7;i--)
			g.drawLine(x[i],y[i],x[i+1],y[i+1]);
		g.setColor(getTabColor().brighter());
		for(i=0;i<5;i++) {
			g.drawLine(x[i],y[i],x[i+1],y[i+1]);
			if (tabNum == selected)
				g.drawLine(x[11-i]-1,y[11-i],x[11-i-1]-1,y[11-i-1]);
			}
		g.drawLine(x[5],y[5],x[6],y[6]);
		g.drawLine(x[6],y[6],x[7],y[7]);
		g.drawLine(x[7],y[7],x[8],y[8]);
		if (tabNum == selected) {
			g.drawLine(x[0],y[0],2,y[0]);
			Dimension d = getSize();
			g.drawLine(x[11]-((tabNum==selected)?1:0),y[0],d.width-4,y[0]);
			g.setColor(getTabColor());
		}

		// write the text for the tab
		g.setColor(getForeground());
		String text;
		if (getTabText() != null && tabNum < getTabText().length)
			text = getTabText()[tabNum];
		else
			text = "<not assigned>";
		g.drawString(text, x[5]+hslop, y[0] - (fm.getDescent() + vslop));
		
		// if the tab extends past the last place we can draw
		//   set tooManyTabs
		if (x[11] > bothRect.x)
			tooManyTabs = true;
	}
	g.setFont(currentFont);

	// draw the +/- for moving through tabs
	g.setColor(getTabColor());
	g.fill3DRect(bothRect.x, bothRect.y, bothRect.width-1, bothRect.height-1, true);
	
	g.setColor(getTabColor().brighter());
	g.drawLine(bothRect.x + 3, bothRect.y + 3, bothRect.x + bothRect.width-6, bothRect.y + bothRect.width-6);
	
	g.setColor(getForeground());
	int w2  = bothRect.width/2;
	int w4  = bothRect.width/4;
	int w8  = bothRect.width/8;
	
	g.drawLine(bothRect.x + w2,    bothRect.y + w4,
			 bothRect.x + w2+w4, bothRect.y + w4);
	g.drawLine(bothRect.x + w2+w8, bothRect.y + w8,
	           bothRect.x + w2+w8, bothRect.y + w4+w8);
	g.drawLine(bothRect.x + w8,    bothRect.y + w2+w8,
	           bothRect.x + w4+w8, bothRect.y + w2+w8);
	
	if (tooManyTabs) {
		// blank out area used for right arrow
		g.setColor(getBorderColor());
		g.fillRect(bothRect.x-13, 0, 13, 25);
	
		// draw arrows
		g.setColor(getTabColor());
		g.fillPolygon(leftArrow);
		g.fillPolygon(rightArrow);
		g.setColor(getTabColor().brighter());
		g.drawPolygon(leftArrow);
		g.drawPolygon(rightArrow);
	}	
}
/**
 * Gets the borderColor property (java.awt.Color) value.
 * The borderColor is the color to paint behind the tabs and around
 *   the edge of the panel display area.
 * @return The borderColor property value.
 * @see #setBorderColor
 */
public java.awt.Color getBorderColor() {
	/* Returns the borderColor property value. */
	if (fieldBorderColor == null) {
		try {
			fieldBorderColor = new java.awt.Color(0);
		} catch (Throwable exception) {
			System.err.println("Exception creating borderColor property.");
		}
	};
	return fieldBorderColor;
}
/**
 * Returns the insets of the container. The insets indicate the size of
 * the border of the container. A Frame, for example, will have a top inset
 * that corresponds to the height of the Frame's title bar. 
 * These insets leave room for the tabs to be drawn at the top of the panel.
 * Note that this should eventually be changed to allow different font sizes
 * @see LayoutManager
 */
public Insets getInsets() {
	return new Insets(29,6,6,6);
}
/**
 * Gets the tabColor property (java.awt.Color) value.
 * The tabColor is the color that will be used to paint the background of
 *   the tabs.
 * @return The tabColor property value.
 * @see #setTabColor
 */
public java.awt.Color getTabColor() {
	/* Returns the tabColor property value. */
	if (fieldTabColor == null) {
		try {
			fieldTabColor = new java.awt.Color(0);
		} catch (Throwable exception) {
			System.err.println("Exception creating tabColor property.");
		}
	};
	return fieldTabColor;
}
/**
 * Gets the tabText property (java.lang.String[]) value.
 * The tabText array represents the text that is drawn onto the tabs.
 * If a tab does not have text assigned to it, "<not assigned>" will be
 * displayed on that tab.
 * @return The tabText property value.
 * @see #setTabText
 */
public java.lang.String[] getTabText() {
	/* Returns the tabText property value. */
	return fieldTabText;
}
/**
 * Gets the tabText index property (java.lang.String) value.
 * The tabText array represents the text that is drawn onto the tabs.
 * If a tab does not have text assigned to it, "<not assigned>" will be
 * displayed on that tab.
 * @return The tabText property value.
 * @param index The index value into the property array.
 * @see #setTabText
 */
public String getTabText(int index) {
	/* Returns the tabText index property value. */
	return getTabText()[index];
}
/**
 * Called whenever the part throws an exception.
 * @param exception java.lang.Throwable
 */
private void handleException(Throwable exception) {

	/* Uncomment the following lines to print uncaught exceptions to stdout */
	// System.out.println("--------- UNCAUGHT EXCEPTION ---------");
	// exception.printStackTrace(System.out);
}
/**
 * Initializes connections
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private void initConnections() {
	// user code begin {1}
	// user code end
	this.addMouseListener(this);
}
/**
 * Initialize class
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
private void initialize() {
	// user code begin {1}
	tabs = new Vector();
	setBorderColor(Color.gray);
	setTabColor(Color.lightGray);
	// user code end
	setName("TabPanel");
	setLayout(new java.awt.CardLayout());
	setBackground(java.awt.Color.lightGray);
	setSize(426, 240);
	initConnections();
	// user code begin {2}
	// user code end
}
/**
 * main entrypoint - starts the part when it is run as an application
 * @param args java.lang.String[]
 */
public static void main(java.lang.String[] args) {
	throw new RuntimeException("TabPanel cannot be run as a 'main'!");
}
/** Merges two tabs' components onto one tab by putting them both on a splitter panel
 *  This needs to be generalized a bit -- right now it assumes that the components in the
 *  spliter panels are definitely TabLabelPanels
 */
protected void mergeTabs(int target, int source) {
	// first things first -- we're no longer dragging a tab...
	tabBeingDragged = -1;

	// deal with the tabs

	// get the text that is displayed on the tabs
	String names[] = getTabText();
	
	// merge the text for the source and target tabs
	//   and remove the source tab from the list of names
	String sourceName = new String(names[source]);
	String targetName = new String(names[target]);
	names[target] += " / " + sourceName;
	for(int i=source; i < names.length-1; i++)
		names[i] = names[i+1];
	if (names.length > 1) names[names.length-1] = null;	
	setTabText(names);
	
	// get the source and target components
	Component t = getComponent(target);
	Component s = getComponent(source);
	
	// remove the source component from the TabSplitter
	remove(s);

   // targetContainer is just a convenience to factor some functionality later
	Component targetContainer = null;
	
  // check to see if one of the subcomponents is splitter panel already...
	boolean tIsSplitter = (t instanceof Container) && (((Container)t).getLayout() instanceof SplitterLayout);
	boolean sIsSplitter = (s instanceof Container) && (((Container)s).getLayout() instanceof SplitterLayout);
	
	if (tIsSplitter) {
		if (sIsSplitter) {
			// BOTH are already splitters
			// Remove the components from the source container
			//   and add them to the target container
			((Container)t).add(new SplitterBar());
			Component cs[] = ((Container)s).getComponents();
			for (int i=0;i<cs.length;i++) {
				((Container)s).remove(cs[i]);
				((Container)t).add(cs[i],new Integer(1));
				cs[i].setVisible(true);
				cs[i].invalidate();
			}	
			targetContainer = t;
		}	

		else {
			// ONLY the target is already a splitter
			// Add the source component to the target container
			TabLabelPanel tlp = new TabLabelPanel(sourceName, s, this);
			((Container)t).add(new SplitterBar());
			((Container)t).add(tlp, new Integer(1));
			targetContainer = t;
		}	
	}	

	else if (sIsSplitter) {
		// ONLY the source is a splitter
		// Add the target at the LEFT side of the source's splitter
		remove(t);
		TabLabelPanel tlp = new TabLabelPanel(targetName, t, this);
		((Container)s).add(new SplitterBar(),0);
		((Container)s).add(tlp, new Integer(1),0);
		targetContainer = s;
		add(s,s.getName(),source<target?target-1:target);
	}	
	
	else {	
		// Neither are splitters
		// Create a new splitter panel
		// Create two new TabLabelPanels
		// Add the target/source to the TabLabelPanels
		// Add the TabLabelPanels to the splitter panel w/ a splitterbar
		remove(t);
		TabLabelPanel tlpt = new TabLabelPanel(targetName, t, this);
		TabLabelPanel tlps = new TabLabelPanel(sourceName, s, this);
		Panel p = new Panel();
		p.setLayout(new SplitterLayout(SplitterLayout.HORIZONTAL));
		p.add(tlpt, new Integer(1));
		p.add(new SplitterBar());
		p.add(tlps, new Integer(1));
		p.setName(t.getName() + "/" + s.getName());
		add(p,p.getName(),source<target?target-1:target);
		targetContainer = p;
	}	

	// make sure the source and target are visible
	s.setVisible(true);	
	t.setVisible(true);	
	// Figure out which tab is selected and show it
	selected = source<target?target-1:target;
	((CardLayout)getLayout()).show(this, targetContainer.getName());

	// Make sure we get repainted/layed out
	s.invalidate();
 	t.invalidate();
 	validate();
	repaint();
}
/**
 * Method to handle events for the MouseListener interface.
 * @param e java.awt.event.MouseEvent
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
public void mouseClicked(java.awt.event.MouseEvent e) {
	// user code begin {1}
	// user code end
	// user code begin {2}
	// user code end
}
/**
 * Method to handle events for the MouseListener interface.
 * @param e java.awt.event.MouseEvent
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
public void mouseEntered(java.awt.event.MouseEvent e) {
	// user code begin {1}
	// user code end
	// user code begin {2}
	// user code end
}
/**
 * Method to handle events for the MouseListener interface.
 * @param e java.awt.event.MouseEvent
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
public void mouseExited(java.awt.event.MouseEvent e) {
	// user code begin {1}
	// user moved mouse out of the tab panel -- abort a drag
	tabBeingDragged = -1;
	setCursor(DEF_CURSOR);
	// user code end
	// user code begin {2}
	// user code end
}
/**
 * Method to handle events for the MouseListener interface.
 * @param e java.awt.event.MouseEvent
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
public void mousePressed(java.awt.event.MouseEvent e) {
	// user code begin {1}
	// check where the mouse was pressed
	if (selected > firstVisible && 
		 ((Polygon)tabs.elementAt(selected-firstVisible)).contains(e.getX(), e.getY()))
		tabBeingDragged = selected;
	else {
		for(int i=firstVisible; i < getComponentCount(); i++)
			if (((Polygon)tabs.elementAt(i-firstVisible)).contains(e.getX(), e.getY())) {
				tabBeingDragged = i;
				break;
				}
		}	
	if (tabBeingDragged != -1) {
		setCursor(HAND_CURSOR);
	}	
	// user code end
	// user code begin {2}
	// user code end
}
/**
 * Method to handle events for the MouseListener interface.
 * @param e java.awt.event.MouseEvent
 */
/* WARNING: THIS METHOD WILL BE REGENERATED. */
public void mouseReleased(java.awt.event.MouseEvent e) {
	// user code begin {1}
	setCursor(DEF_CURSOR);
	// user code end
	if ((e.getSource() == this) ) {
		conn0(e);
	}
	// user code begin {2}
	// user code end
}
/**
 * Move to next tab, wrapping around to first tab if necessary.
 */
public void next( ) {
	((CardLayout)getLayout()).next(this);
	selected = ++selected % getComponentCount();
	if (tooManyTabs) firstVisible = selected;
	repaint();
}
/** Paints the tab panel image (tabs, border, background)
 * @param g (java.awt.Graphics) The graphics context into which the tab panel is painted
 */
public void paint(Graphics g) {
	Dimension d = getSize();
	if ((d.width != lastW) || (d.height != lastH) || (image == null)) {
		// size has changed, must resize image
		image = createImage(d.width, d.height);
		if (g1 != null) g1.dispose();
		g1  = image.getGraphics();

		lastW = d.width;
		lastH = d.height;
		}

	g1.setColor(getBorderColor());
	g1.fillRect(0,0,d.width,d.height);
	g1.setColor(getBackground());
	g1.fill3DRect(2,25,d.width-4,d.height-27,true);
	g1.setColor(getTabColor());
	g1.draw3DRect(4,27,d.width-8,d.height-31,false);
	drawTabs(g1);

	g.drawImage(image,0,0,this);
	super.paint(g);
}
/**
 * Move to previous tab, wrapping around to last tab if necessary.
 */
public void previous() {
	((CardLayout)getLayout()).previous(this);
	if (--selected < 0) selected = getComponentCount()-1;
	if (tooManyTabs) firstVisible = selected;
	repaint();
}
/** Remove a component from the container.
 *  This method is overridden to force a repaint in design mode.
 * @param index (int) Which component to remove.
 */
public void remove(int index) {
	Component c = getComponent(index);
	super.remove(index);
	if (java.beans.Beans.isDesignTime())
		repaint();
}
/** Remove a component from the container.
 *  This method is overridden to force a repaint in design mode.
 * @param comp (java.awt.Component) Which component to remove.
 */
public void remove(Component comp) {
	super.remove(comp);
	if (java.beans.Beans.isDesignTime())
		repaint();
}
/** Remove all components from the container.
 *  This method is overridden to force a repaint in design mode.
 */
public void removeAll() {
	super.removeAll();
	if (java.beans.Beans.isDesignTime())
		repaint();
}
/**
 * Check to see if thge user clicked on a tab or one of the
 * tab panel navigation buttons.
 * @param e (java.awt.event.MouseEvent) The mouse click.
 */
protected void selectTab(MouseEvent e) {
	// walk through the tab polygons to see if we have a hit...
	// first check if it's the "plus" or "minus"
	if (bothRect.contains(e.getX(), e.getY())) {
		if (e.getX() - bothRect.x > e.getY() - bothRect.y) // plus
			next();
		else // minus
			previous();
	}	

  // if the user clicked on the right arrow, shift the tabs to the right
	else if (tooManyTabs && rightArrow.contains(e.getX(), e.getY()))
		shiftRight();
	
	// if the user clicked on the left arrow, shift the tabs to the left
	else if (tooManyTabs && leftArrow.contains(e.getX(), e.getY()))
		shiftLeft();

	// In this section, we check to see if the user clicked on a tab in the panel.	
	// first, check the "selected" one -- it's at front
	else if (selected >= firstVisible && ((Polygon)tabs.elementAt(selected-firstVisible)).contains(e.getX(), e.getY())) {
		// (condition means "selected tab is visible and they released the mouse while in it.)
		if (tabBeingDragged != selected)          // if they were dragging a tab
			mergeTabs(selected, tabBeingDragged);  // merge the source tab with the selected tab
		// else -- it's already selected, so do nothing
	}	

	// now walk through the rest of the tabs and see if the user clicked in one of them
	// the extra check is to filter out tests for the currently-selected tab
	else if (selected < firstVisible || !((Polygon)tabs.elementAt(selected-firstVisible)).contains(e.getX(), e.getY())) {
		// for each tab starting at the first visible tab, moving right
		for(int i=firstVisible; i < getComponentCount(); i++)
			if (((Polygon)tabs.elementAt(i-firstVisible)).contains(e.getX(), e.getY()))
			
				// if they released on the same tab they pressed, or nothing was being dragged
				if (tabBeingDragged == i || tabBeingDragged == -1) {
					// select the target tab
					selected = i;
					Component comp[] = getComponents();
					((CardLayout)getLayout()).show((Container)this, comp[i].getName());
					repaint();
					break;
					}
					
				// otherwise, they were dragging a tab and released on a different tab -- merge them
				else
					mergeTabs(i, tabBeingDragged);
		}
}
/** separateTabs -- split the selected tab out from its parent Splitterlayout panel
 */
public void separateTabs(TabLabelPanel labelPanel) {
	Container parent = labelPanel.getParent();
	Component tabComps[] = getComponents();
	
	// find the parent panel
	int parentNum = 0;
	while(tabComps[parentNum] != parent) parentNum++;
	// parentNum now points to the parent component	(allow throw of arraybounds)
	
	// find the labelPanel within it
	Component splitterComps[] = parent.getComponents();
	int childNum = 0;
	while(splitterComps[childNum] != labelPanel) childNum++;

	// if there is a splitterbar before it, remove it
	if (childNum > 0 && (splitterComps[childNum-1] instanceof SplitterBar))
		parent.remove(splitterComps[childNum-1]);
	// else if there is a splitterbar after it, remove it
	else if (childNum < (splitterComps.length-1) && (splitterComps[childNum+1] instanceof SplitterBar))
		parent.remove(splitterComps[childNum+1]);
	
	// remove the labelPanel from its parent
	parent.remove(labelPanel);
	
	// remove the contents of the labelPanel & get the panel name
	Component thing = labelPanel.removeComponent();
	String tabName = labelPanel.getTabName();
	
	// add the contents to the tab panel with the panel name as the tab
	add(thing.getName(), thing);
	selected = getComponentCount()-1;
	setTabText(selected, tabName);
	thing.invalidate();
	((CardLayout)getLayout()).show(this, thing.getName());
	
	// if there is only one component left in the parent
	if (parent.getComponentCount() == 1) {
		// remove the component from the parent
		labelPanel = (TabLabelPanel)parent.getComponent(0);
		parent.remove(labelPanel);
		
		// remove the parent from the tabPanel
		remove(parent);
		
		// add the component to the tab panel in its old spot
		thing = labelPanel.getComponent();
		thing.invalidate();
		tabName = labelPanel.getTabName();
		add(thing, thing.getName(), parentNum);
		setTabText(parentNum, tabName);
	}	
	
	// there is still more than one component in the splitter
	else {
		String newTabText = "";
		// set the tab text appropriately
		splitterComps = parent.getComponents();
		for (int i=0;i<splitterComps.length; i++)
			if (splitterComps[i] instanceof TabLabelPanel)
				newTabText += ((TabLabelPanel)splitterComps[i]).getTabName();
			else
				newTabText += " / ";
		setTabText(parentNum, newTabText);
	}	
	repaint();
}	
/**
 * Sets the borderColor property (java.awt.Color) value.
 * The borderColor is the color to paint behind the tabs and around
 *   the edge of the panel display area. 
 * @param borderColor The new value for the property.
 * @see #getBorderColor
 */
public void setBorderColor(java.awt.Color borderColor) {
	fieldBorderColor = borderColor;
	return;
}
/**
 * Sets the tabColor property (java.awt.Color) value.
 * @param tabColor The new value for the property.
 * @see #getTabColor
 */
public void setTabColor(java.awt.Color tabColor) {
	fieldTabColor = tabColor;
	return;
}
/**
 * Sets the tabText property (java.lang.String[]) value.
 * The tabText array represents the text that is drawn onto the tabs.
 * If a tab does not have text assigned to it, "<not assigned>" will be
 * displayed on that tab.
 * @param tabText The new value for the property.
 * @see #getTabText
 */
public void setTabText(java.lang.String[] tabText) {
	fieldTabText = tabText;
	return;
}
/**
 * Sets the tabText index property (java.lang.String[]) value.
 * The tabText array represents the text that is drawn onto the tabs.
 * If a tab does not have text assigned to it, "&lt;not assigned&gt;" will be
 * displayed on that tab.
 * @param index The new value for the property.
 * @param tabText The new value for the property.
 * @see #getTabText
 */
public void setTabText(int index, String tabText) {
	fieldTabText[index] = tabText;
	return;
}
/** Creates a Polygon object for each tab.  This polygon object
 *  is used to draw the tab and to test if the user has clicked
 *  the mouse within that tab.   The navigation buttons are also
 *  set up in this method.
 * @param g (java.awt.Graphics) The graphics context into which the tabs will be drawn
 */
protected final void setupTabPolygons(Graphics g) {
	Dimension dim = getSize();
	int x[] = new int[6];
	int y[] = new int[12];

	bothRect  = new Rectangle(dim.width-26, 2, 24, 24);
	x[0] = 2;  x[1] = 11; x[2] = 11;
	y[0] = 11; y[1] = 7;  y[2] = 15;
	leftArrow = new Polygon(x,y, 3);
	x[0] = bothRect.x - x[0];
	x[1] = bothRect.x - x[1];
	x[2] = bothRect.x - x[2];
	rightArrow = new Polygon(x,y, 3);
	
	FontMetrics fm = g.getFontMetrics();
	int i;
	x[0]=0;  x[1]=3;  x[2]=4;  x[3]=9; x[4]=11; x[5]=14;
	y[0]=25; y[1]=23; y[2]=22; y[3]=5; y[4]=4;  y[5]=2;

	for(i=0; i<6; i++) y[11-i] = y[i];

	int x1[] = new int[12];
	g.setColor(Color.lightGray);
		
	// should try to resize tabs based on font metrics...

	Font currentFont = g.getFont();
//		Font currentFont = new Font("Serif",Font.PLAIN, 12); g.setFont(currentFont);
	Font boldFont = new Font(currentFont.getName(), Font.BOLD, currentFont.getSize());
	int tempXOff = 8;
	Component comp[] = getComponents();
	tabs.removeAllElements();
	for(int num=firstVisible; num<comp.length; num++) {
		if (num == selected) { // switch to bold font for metrics
			g.setFont(boldFont);
			fm = g.getFontMetrics();
			}

		String text;
		if (getTabText() != null && num < getTabText().length)
			text = getTabText()[num];		
		else
			text = "<not assigned>";
		int textWidth = fm.stringWidth(text);
			
		for(i=0; i<6; i++)
			x1[i] = x[i] + tempXOff;
			
		tempXOff += ((x[5]-x[0])*2) + (hslop*2) + textWidth;
		
		for(i=0; i<6; i++)
			x1[11-i] = tempXOff - x[i];
			
		tempXOff -= x[5];
		
		if (num == selected) { // switch back
			g.setFont(currentFont);
			fm = g.getFontMetrics();
			}
		Polygon p = new Polygon(x1,y,12);
		tabs.addElement(p);
		}
}
/** Shifts the row of tabs one position to the left
 */
public void shiftLeft() {
	if (firstVisible > 0) firstVisible--;
	repaint();
}	
/** Shifts the row of tabs one position to the right
 */
public void shiftRight() {
	if (firstVisible < (getComponentCount()-1)) firstVisible++;
	repaint();
}
}