import com.ibm.xml.parser.DTD;
import com.ibm.xml.parser.ElementDecl;
import com.ibm.xml.parser.FormatPrintVisitor;
import com.ibm.xml.parser.InsertableElement;
import com.ibm.xml.parser.NonRecursivePreorderTreeTraversal;
import com.ibm.xml.parser.Parser;
import com.ibm.xml.parser.Stderr;
import com.ibm.xml.parser.TXElement;
import com.ibm.xml.parser.TXText;
import com.ibm.xml.parser.Visitor;
import java.awt.Button;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.List;
import java.awt.TextArea;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.StringWriter;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;


public class GeneratingSample extends WindowAdapter implements Runnable
{
static final String S_EOC="********End OF Content *******";
static final String S_TEXT="******* any text *********";

public static void main(String[] argv)
{
	new GeneratingSample(argv);
}

DTD dtd;
Hashtable hash;
Frame frame;
Vector allElement;
TextArea text;
TXElement root;

GeneratingSample(String[] argv)
{
	try
	{
	String fname=null;
	for(int i=0;i<argv.length;i++)
	{
		if(argv[i].charAt(0)!='-')
		{
			fname=argv[i];
		}
		else
		{
			System.err.println("Warning:Unknown Option"+argv[i]);
		}	
	}		

	Parser pc=new Parser(fname);
	String targeturl=Stderr.file2URL(fname).toString();
	this.dtd=pc.readDTDStream(pc.getInputStream(fname,null,targeturl));
	if(this.dtd==null)
	{
		System.err.println("no DTD");
	}
	else
	{
		this.allElement=new Vector();
		Enumeration en=this.dtd.getElementDeclarations();
		while(en.hasMoreElements())
		{
			ElementDecl ed=(ElementDecl)en.nextElement();
			this.allElement.addElement(ed.getName());
		}
		this.hash=this.dtd.prepareTable((String)this.allElement.elementAt(0));
		en=this.allElement.elements();
		while(en.hasMoreElements())
		{
			String n=(String)en.nextElement();
			this.hash.put(n,new InsertableElement(n));
		}
		this.frame=new Frame("Generating Sample");
		this.frame.addWindowListener(this);
		this.frame.add(this.text=new TextArea(),"Center");
		this.frame.pack();
		centerComponentInScreen(this.frame,384,220);
		this.frame.show();
	}
	}
	catch(Exception e)
	{
		e.printStackTrace();
	}
}

public static void centerComponent(Component com,int w,int h,Dimension di)
{
	com.setSize(w,h);
	com.setLocation((di.width-w)/2,(di.height-h)/2);
}

public static void centerComponentInScreen(Component com,int w,int h)
{
	centerComponent(com,w,h,Toolkit.getDefaultToolkit().getScreenSize());	
}

public void windowOpened(WindowEvent e)
{
	new Thread(this).start();
}

public void windowClosing(WindowEvent e)
{
	this.frame.dispose();
	System.exit(0);
}

void out()
{
	try
	{
		StringWriter sw=new StringWriter();
		Visitor v=new FormatPrintVisitor(sw);
		new NonRecursivePreorderTreeTraversal(v).traverse(this.root);
		sw.close();
		this.text.setText(sw.toString());
	}
	catch(Exception e)
	{
		e.printStackTrace();
	}
}

public void run()
{
	this.text.setText("Select root element name from the list on the dialog");
	String rootname=choice(this.frame,"Select root element",sortStringVector(this.allElement));
	this.root=new TXElement(rootname);
	this.out();
	this.construct(this.root);
	System.err.println("Complete.");
}

void construct(TXElement parent)
{
	switch(this.dtd.getContentType(parent.getNodeName()))
	{
		case -1:
			System.err.println("This element "+parent.getNodeName()+" is not decided.");
			break;
		case ElementDecl.EMPTY:
                        System.err.println("This element "+parent.getNodeName()+" is declared as empty.");
   			break;
		case ElementDecl.ANY:
                        System.err.println("This element "+parent.getNodeName()+" is declared as any.");
			break;                        
		case ElementDecl.MODEL_GROUP:
			while(true)
			{
				Vector vec=makeAvailable(this.dtd,parent,this.hash);
				if(vec.size()==0)
					break;
				if(vec.size()==1 && S_EOC.equals((String)vec.elementAt(0)))
					break;
				vec=sortNames(this.dtd.makeContentElementList(parent.getNodeName()),vec);
				String chi=choice(this.frame,"Select next child of "+parent.getNodeName(),vec);
				if(chi.equals(DTD.CM_PCDATA) || chi.equals(S_TEXT))
				{
					parent.appendChild(new TXText("Some Text"));
					this.out();
				}
				else if(chi.equals(S_EOC))
				{
					break;
				}
				else
				{
					TXElement child=new TXElement(chi);
					parent.appendChild(child);
					this.out();
					this.construct(child);
				}
			}
			break;
		default:
			System.err.println("internal Error.");
	}
	
}

static Vector makeAvailable(DTD dtd,TXElement el,Hashtable hash)
{
	Vector v=new Vector();
	dtd.getAppendableElements(el,hash);
	if(((InsertableElement)hash.get(DTD.CM_ERROR)).status)
	{
		System.err.println("elemnet hsa wrong structure.");
	}
	else
	{
		Enumeration en=hash.elements();
		while(en.hasMoreElements())
		{
			InsertableElement ie=(InsertableElement)en.nextElement();
			if(!ie.name.equals(DTD.CM_ERROR) && ie.status)
			{
				if(ie.name.equals(DTD.CM_EOC))
					v.addElement(S_EOC);
				else
					v.addElement(ie.name);
			}
		}
	}
	return v;
}

static String choice(Frame parent,String tit,Vector v)
{
	final Dialog f=new Dialog(parent,tit,true);
	final List list=new List();
	Enumeration en=v.elements();
	while(en.hasMoreElements())
	{
		String el=(String)en.nextElement();
		list.add(el.equals(DTD.CM_PCDATA) ? S_TEXT:el);
	}
	
	Button btok=new Button("OK");
	f.add(list,"Center");
	f.add(btok,"South");
	btok.addActionListener(new ActionListener()
	{ public void actionPerformed(ActionEvent e)
	  {
	  	if(null!=list.getSelectedItem())
			f.setVisible(false);
		else
			System.err.println("Select 1 item !");
	  }
	 });
	f.pack();
	centerComponentInScreen(f,256,220);
	f.show();

	String selected=list.getSelectedItem();
	f.dispose();
	System.err.println("SELECTED: "+selected);
	return selected;
}


static private void fall(String[] pd,int n,int i)
{
	int j=2*i+1;
	if(j<n)
	{
		if(j+1 < n)
		{
			if(0 > pd[j].compareTo(pd[j+1]))
				j=2*i+2;
		}
		else
		{}
		if(pd[i].compareTo(pd[j]) <0)
		{
			String t=pd[i];
			pd[i]=pd[j];
			pd[j]=t;
			fall(pd,n,j);
		}
	}
	
}

static void heapSort(String[] pd)
{
	int i;
	for(i=pd.length/2;i>=0;i--)
	{
		fall(pd,pd.length,i);
	}
	for(i=pd.length-1;i>0;i--)
	{
		String t=pd[0];
		pd[0]=pd[i];
		pd[i]=t;
		fall(pd,i,0);
	}
}


static Vector sortStringVector(Vector v)
{
	String[] as=new String[v.size()];
	v.copyInto(as);
	heapSort(as);
	v.removeAllElements();
	v.ensureCapacity(as.length);
	for(int i=0;i<as.length;i++)
		v.addElement(as[i]);
	return v;
}


static Vector sortNames(Vector base,Vector v)
{
	if(null==base)
	{
		return sortStringVector(v);
	}
	Vector ret=new Vector();
	for(int i=0;i<base.size();i++)
	{
		String n=(String)base.elementAt(i);
		int ind=v.indexOf(n);
		if(ind >=0)
		{
			ret.addElement(n);
			v.removeElementAt(ind);
		}
	}
	for(int i=0;i<v.size();i++)
	{
		ret.addElement(v.elementAt(i));
	}
	return ret;
}
}

			
		
			


	
				
					
