/*
 * @(#)Arcs.java	1.11 98/06/29
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package demos.Arcs_Curves;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Arc2D;
import java.awt.geom.AffineTransform;
import J2DCanvas;


/**
 * Arc2D Open, Chord & Pie arcs; Animated Pie Arc.
 */
public class Arcs extends J2DCanvas {

    private int x, y;
    private Arc2D.Float arcs[] = { new Arc2D.Float(Arc2D.CHORD),
			           new Arc2D.Float(Arc2D.OPEN),
				   new Arc2D.Float(Arc2D.PIE) };
    private String types[] = { "Arc2D.CHORD", "Arc2D.OPEN", "Arc2D.PIE" };

    private int ah, aw;   // sizes for animated pie arc
    private int angleStart = 45;
    private int angleExtent = 270;
    private final int CLOSE = 0;
    private final int OPEN = 1;
    private final int FORWARD = 0;
    private final int BACKWARD = 1;
    private final int DOWN = 2;
    private final int UP = 3;
    private int mouth = CLOSE;
    private int direction = FORWARD;


    public Arcs() {
	setBackground(Color.white);
	runnable = true;
    }


    public void drawDemo(Graphics2D g2) {

        if (newBufferedImage) {
            aw = w/12;
            ah = h/12;
            mouth = CLOSE;
            angleStart = 45;
            angleExtent = 270;
            direction = FORWARD;
            x = 0;
        }

	g2.setStroke(new BasicStroke(5.0f));
      // Draw Arcs
	for (int i = 0; i < arcs.length; i++) {
	    arcs[i].setFrame((i+1)*w/5, (i+1)*h/5, w/6, h/6);
	    arcs[i].setAngleStart(45);
	    arcs[i].setAngleExtent(270);
	    g2.setColor(Color.blue);
	    g2.draw(arcs[i]);
	    g2.setColor(Color.gray);
	    g2.fill(arcs[i]);
	    g2.setColor(Color.black);
	    g2.drawString(types[i], (i+1)*w/5, (i+1)*h/5-3);
	}

      // Draw Animated Pie Arc
	arcs[2].setFrame(0, 0, aw, ah);
	arcs[2].setAngleStart(angleStart);
	arcs[2].setAngleExtent(angleExtent);
        AffineTransform at = new AffineTransform();
        at.translate(x,y);
        switch (direction) {
            case DOWN : at.rotate((90*java.lang.Math.PI)/180); break;
            case BACKWARD : at.rotate((180*java.lang.Math.PI)/180); break;
            case UP : at.rotate((270*java.lang.Math.PI)/180);
        }
        g2.transform(at);
        g2.setColor(Color.blue);
        g2.fill(arcs[2]);
        at.setToIdentity();
        g2.setTransform(at);

      // Compute direction
        if (x+aw >= w-5 && direction == FORWARD)
            direction = DOWN;
        if (y+ah >= h-5 && direction == DOWN)
            direction = BACKWARD;
        if (x-aw <= 5 && direction == BACKWARD)
            direction = UP;
        if (y-ah <= 5 && direction == UP)
            direction = FORWARD;

      // compute angle start & extent
        if (mouth == CLOSE) {
            angleStart -= 5;
            angleExtent += 10;
        }
        if (mouth == OPEN) {
            angleStart += 5;
            angleExtent -= 10;
        }
        if (direction == FORWARD) {
            x += 5; y = 0;
        }
        if (direction == DOWN) {
            x = w; y += 5;
        }
        if (direction == BACKWARD) {
            x -= 5; y = h;
        }
        if (direction == UP) {
            x = 0; y -= 5;
        }
        if (angleStart == 0)
            mouth = OPEN;
        if (angleStart > 45)
            mouth = CLOSE;
    }


    public static void main(String argv[]) {
	final Arcs demo = new Arcs();
	WindowListener l = new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	    public void windowDeiconified(WindowEvent e) { demo.start(); }
	    public void windowIconified(WindowEvent e) { demo.stop(); }
	};
	Frame f = new Frame("Java2D Demo - Arcs");
	f.addWindowListener(l);
	f.add("Center", demo);
	f.pack();
	f.setSize(new Dimension(400,300));
	f.show();
	demo.start();
    }
}
