/*
 * @(#)CMM.java	1.20 98/06/29
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package demos.Colors;

import java.awt.*;
import java.awt.event.*;
import java.awt.color.ICC_Profile;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.awt.font.TextLayout;
import java.awt.font.FontRenderContext;
import java.io.IOException;
import java.net.URL;
import J2DCanvas;


/**
 * ColorConvertOp with ICC_Profile's on a BufferedImage of a image, text
 * and colored rectangles.
 */
public class CMM extends J2DCanvas {


    private Image img;

    public CMM() {
	setBackground(Color.white);
        img = getImage("images/clouds.jpg");
    }


    public void drawDemo(Graphics2D g2) {

	int iw = img.getWidth(this);
	int ih = img.getHeight(this);

	FontRenderContext frc = g2.getFontRenderContext();
	g2.setColor(Color.black);
	TextLayout tl = new TextLayout("ColorConvertOp w/ ICC_Profile", font, frc);
	tl.draw(g2, (float) (w/2-tl.getBounds().getWidth()/2),
			    tl.getAscent()+tl.getLeading());

        BufferedImage srcImg = new BufferedImage(iw, ih, BufferedImage.TYPE_INT_RGB);
        Graphics2D srcG = srcImg.createGraphics();
        srcG.drawImage(img, 0, 0, null);

	tl = new TextLayout("Java2D", new Font("Times Bold", Font.PLAIN, iw/6), frc);
	float ssw = (float) tl.getBounds().getWidth();
	srcG.setColor(Color.blue);
	tl.draw(srcG, (float) (iw/2-ssw/2), (float) (ih-25));

	Color colors[] = { Color.red, Color.green, Color.blue };
	for (int i = 0; i < 3; i++) {
	    srcG.setColor(colors[i]);
	    srcG.fill(new Rectangle(i*(iw/3),ih-20,iw/3,ih));
	}

	BufferedImage destImg;
	String msg;
	try {
	    URL url = CMM.class.getResource("src.pf");
	    ICC_Profile pf1 = ICC_Profile.getInstance(url.getFile());
	    url = CMM.class.getResource("dest.pf");
	    ICC_Profile pf2 = ICC_Profile.getInstance(url.getFile());
	    ICC_Profile pfList[] = new ICC_Profile[2];
	    pfList[0] = pf1;
	    pfList[1] = pf2;
	    ColorConvertOp theOp = new ColorConvertOp(pfList, null);

	    destImg = new BufferedImage(iw, ih, BufferedImage.TYPE_INT_RGB);
	    theOp.filter(srcImg, destImg);
	    msg = "Converted";
	} catch (IOException e) {
	    msg = "Missing profile";
	    destImg = null;
	}

        g2.drawImage(srcImg, 10, 20, w/2-20, h-30, this);
	g2.drawString("Original", 12, 31);

	if (destImg != null) {
	    g2.drawImage(destImg, w/2+10, 20, w/2-20, h-30, this);
	    g2.drawString(msg, w/2+12, 31);
	}
    }


    public static void main(String s[]) {
	WindowListener l = new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	};
	Frame f = new Frame("Java2D Demo - CMM");
	f.addWindowListener(l);
	f.add("Center", new CMM());
	f.pack();
	f.setSize(new Dimension(400,300));
	f.show();
    }
}
