/*
 * @(#)ObjectAnim.java	1.17 98/06/29
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package demos.Transforms;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.awt.font.TextLayout;
import java.awt.font.FontRenderContext;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import J2DCanvas;


/**
 * Animation of shapes, text and images rotating, scaling and translating
 * around a canvas.
 */
public class ObjectAnim extends J2DCanvas {

    private double[] xx, yy;
    private Object[] objects;
    private final double OINC[] = {5.0, 3.0};
    private final double SINC[] = {5.0, 5.0};
    private double[] ix, iy;
    private Color colors[] = { Color.blue, Color.cyan, Color.green,
			Color.magenta, Color.orange, Color.pink,
			Color.red, Color.yellow, Color.lightGray,
			Color.white };
    private int rotate[];
    private double scale[];
    private final int UP = 0;
    private final int DOWN = 1;
    private int scaleDirection[];
    private Image imgs[];
    private int imgN;


    public ObjectAnim() {

       	setBackground(Color.black);

	objects = new Object[18];
	objects[0] = getImage("images/jumptojavastrip.gif");
        imgs = new Image[10];
        MediaTracker tracker = new MediaTracker(this);
        for (int i=0, x=0; i < 10; i++, x+=80) {
            CropImageFilter cif = new CropImageFilter(x, 0, 80, 80);
            ImageProducer ip = ((Image) objects[0]).getSource();
            ip = new FilteredImageSource(ip, cif);
            imgs[i] = getToolkit().createImage(ip);
            tracker.addImage(imgs[i], i);
        }
	try {
            tracker.waitForAll();
	} catch (Exception e) { e.printStackTrace(); }

	objects[1] = new Rectangle2D.Float(0,0,30,60);
	objects[2] = new Ellipse2D.Float(0,0,30,30);
	objects[3] = new Arc2D.Float(0,0,35,35,45,270, Arc2D.PIE);
	objects[4] = getImage("images/duke.gif");
	objects[5] = null;
	objects[6] = null;
	objects[7] = getImage("images/HotJava-16.gif");
	objects[8] = new QuadCurve2D.Float(0,0,20,60,60,0);
	objects[9] = null;
	objects[10] = new CubicCurve2D.Float(0,0,30,-60,60,60,90,0);
	objects[11] = new RoundRectangle2D.Float(0,0,40,40,10,10);


	rotate = new int[objects.length];
	for (int i = 0; i < objects.length; i++) {
	    rotate[i] = (int)(Math.random() * 360);
	}

	scale = new double[objects.length];
	scaleDirection = new int[objects.length];
	for (int i = 0; i < objects.length; i++) {
	    scale[i] = Math.random() * 1.5;
	    scaleDirection[i] = ((i%2) == 0) ? UP : DOWN;
	}

	xx = new double[objects.length];
	yy = new double[objects.length];
        ix = new double[objects.length];
        iy = new double[objects.length];

	for (int i = 0; i < objects.length; i++ ) {
    	    ix[i] = OINC[0];
    	    iy[i] = OINC[1];
	}

	runnable = true;
    }


    public void drawDemo(Graphics2D g2) {

	if (newBufferedImage) {
	    for (int i = 0; i < objects.length; i++ ) {
       	        xx[i] = Math.random()*w;
       	        yy[i] = Math.random()*h;
	    }
	    FontRenderContext frc = g2.getFontRenderContext();
	    Font f = new Font("Times New Roman", Font.PLAIN, 32);
	    TextLayout tl = new TextLayout("Transformations", f, frc);
	    objects[5] = tl.getOutline(null, 0, 0);
	    f = new Font("serif", Font.BOLD + Font.ITALIC, 24);
	    tl = new TextLayout("Rotate", f, frc);
	    objects[6] = tl.getOutline(null, 0, 0);
	    f = new Font("Helvetica", Font.PLAIN, 24);
	    tl = new TextLayout("Scale", f, frc);
	    objects[9] = tl.getOutline(null, 0, 0);
	    objects[12] = new Ellipse2D.Float(0,0,w/8,w/8);
	    objects[13] = new Rectangle2D.Float(0,0,w/10,w/10);
	    objects[14] = new RoundRectangle2D.Float(0,0,w/8,h/8,10,10);
	    objects[15] = new Arc2D.Float(0,0,w/7,w/7,45,270, Arc2D.PIE);
	    f = new Font("Times New Roman", Font.ITALIC, w/8);
	    tl = new TextLayout("Translate", f, frc);
	    objects[16] = tl.getOutline(null, 0, 0);
	    objects[17] = new CubicCurve2D.Float(0,0,w/12,-w/2,w/6,w/2,w/3,0);
        }

	for (int i = 0; i < objects.length; i++) {
	    if (objects[i] == null)
		continue;
            xx[i] += ix[i];
            yy[i] += iy[i];
            if (xx[i] > w) {
                xx[i] = w - 1;
	 	ix[i] = Math.random() * -w/32 - 1;
            }
            if (xx[i] < 0) {
                xx[i] = 2;
	 	ix[i] = Math.random() * w/32 + 1;
            }
            if (yy[i] > h ) {
                yy[i] = h-2;
	 	iy[i] = Math.random() * -h/32 - 1;
            }
            if (yy[i] < 0) {
                yy[i] = 2;
	 	iy[i] = Math.random() * h/32 + 1;
            }
	    AffineTransform at = new AffineTransform();
	    at.translate(xx[i], yy[i]);
	    at.rotate((rotate[i]*java.lang.Math.PI)/180);
	    at.scale(scale[i],scale[i]);
	    g2.setTransform(at);
            if (objects[i] instanceof Image) {
		 if (i == 0)
                      objects[i] = imgs[(imgN++)%imgs.length];
        	g2.drawImage((Image) objects[i], 0, 0, this);
	    } else {
	        g2.setColor(colors[i%colors.length]);
		if (objects[i] instanceof QuadCurve2D || objects[i] instanceof CubicCurve2D) {
        	    g2.setStroke(new BasicStroke(3.0f));
                    g2.draw((Shape) objects[i]);
		} else
                    g2.fill((Shape) objects[i]);
	    }
	    at.setToIdentity();
	    g2.setTransform(at);

	    if ((rotate[i]+=5) == 360) {
		rotate[i] = 0;
	    }
	    if (scaleDirection[i] == UP) {
		if ((scale[i] += 0.05) > 1.5)
		    scaleDirection[i] = DOWN;
	    } else if (scaleDirection[i] == DOWN) {
		if ((scale[i] -= .05) < 0.5)
		    scaleDirection[i] = UP;
	    }
	}
    }


    public static void main(String s[]) {
	final ObjectAnim demo = new ObjectAnim();
	WindowListener l = new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	    public void windowDeiconified(WindowEvent e) { demo.start(); }
	    public void windowIconified(WindowEvent e) { demo.stop(); }
	};
	Frame f = new Frame("Java2D Demo - ObjectAnim");
	f.addWindowListener(l);
	f.add("Center", demo);
	f.pack();
	f.setSize(new Dimension(400,300));
	f.show();
	demo.start();
    }
}
