/*
 * @(#)Shear.java	1.15 98/06/29
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package demos.Transforms;

import java.awt.*;
import java.awt.event.*;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import J2DCanvas;


/**
 * Shearing a image animated.
 */
public class Shear extends J2DCanvas {

    private int iw,ih;
    private Image img;
    private double sx, sy;
    private int shearDir;
    private final int RIGHT = 0;
    private final int LEFT = 1;
    private final int XMIDDLE = 2;
    private final int DOWN = 3;
    private final int UP = 4;
    private final int YMIDDLE = 5;
    private final int XupYup = 6;
    private final int XdownYdown = 7;


    public Shear() {
        setBackground(Color.white);
	String s = "images/painting.gif";
	img = getImage(s);
	iw = img.getWidth(this);
	ih = img.getHeight(this);
	runnable = true;
    }


    public void drawDemo(Graphics2D g2) {

	if (newBufferedImage) {
	    sx = 0;
	    sy = 0;
	    shearDir = RIGHT;
	}

	AffineTransform at = new AffineTransform();
	at.translate(w/2-iw/2,h/2-ih/2);
	at.shear(sx, sy);

	g2.setColor(Color.orange);
	Rectangle border = new Rectangle(-5,-5,iw+10,ih+10);
	Shape shape = at.createTransformedShape(border);
        g2.fill(shape);

        g2.drawImage(img, at, this);

	g2.setColor(Color.black);
	TextLayout tl = new TextLayout("Shear", g2.getFont(), g2.getFontRenderContext());
	tl.draw(g2, (float) (w/2-tl.getBounds().getWidth()/2),
			    tl.getAscent()+tl.getLeading());

	String s = Double.toString(sx);
	s = (s.length() < 5) ? s : s.substring(0,5);
	tl = new TextLayout("sx=" + s, g2.getFont(), g2.getFontRenderContext());
	float ssw = (float) (tl.getBounds().getWidth()+4);
	tl.draw(g2, 2,h-4);

	s = Double.toString(sy);
	s = (s.length() < 5) ? s.substring(0,s.length()) : s.substring(0,5);
	tl = new TextLayout("sy=" + s, g2.getFont(), g2.getFontRenderContext());
	tl.draw(g2, ssw,h-4);

	double sw = shape.getBounds().getWidth();
	double sh = shape.getBounds().getHeight();
	if (shearDir == RIGHT) {
	    sx += .05;
	    if (sw > w/2+iw/2) {
	        shearDir = LEFT;
	        sx -= .1;
	    }
	} else if (shearDir == LEFT) {
            sx -= .05;
	    if (sw > w/2+iw/2)
	        shearDir = XMIDDLE;
	} else if (shearDir == XMIDDLE) {
	    sx += .05;
	    if (sx > 0) {
	        shearDir = DOWN;
	        sx = 0;
	    }
	} else if (shearDir == DOWN) {
	    sy -= .05;
	    if (sh > h/2+ih/2) {
	        shearDir = UP;
	        sy += .1;
	    }
        } else if (shearDir == UP) {
            sy += .05;
	    if (sh > h/2+ih/2)
	        shearDir = YMIDDLE;
        } else if (shearDir == YMIDDLE) {
            sy -= .05;
            if (sy < 0) {
	        shearDir = XupYup;
	        sy = 0;
	    }
        } else if (shearDir == XupYup) {
            sx += .05; sy += .05;
	    if (sh > h/2+ih/2)
	        shearDir = XdownYdown;
        } else if (shearDir == XdownYdown) {
            sy -= .05; sx -= .05;
            if (sy < 0) {
	        shearDir = RIGHT;
	        sx = sy = 0;
	    }
	}
    }


    public static void main(String s[]) {
	final Shear demo = new Shear();
	WindowListener l = new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	    public void windowDeiconified(WindowEvent e) { demo.start(); }
	    public void windowIconified(WindowEvent e) { demo.stop(); }
	};
	Frame f = new Frame("Java2D Demo - Shear");
	f.addWindowListener(l);
	f.add("Center", demo);
	f.pack();
	f.setSize(new Dimension(400,300));
	f.show();
	demo.start();
    }
}
