// Copyright  1998 Sun Microsystems, Inc. All Rights Reserved.
//
package examples.activation; 

import java.rmi.*;
import java.rmi.activation.*;
import java.util.Properties;

public class Setup {

    // This class registers information about the ActivatableImplementation
    // class with rmid and the rmiregistry
    //
    public static void main(String[] args) throws Exception {

	MyRemoteInterface mri;  // Declare an instance of the interface

	System.setSecurityManager(new RMISecurityManager());

	// The "location" String specifies a URL from where the class   
	// definition will come when this object is requested (activated).
        // Don't forget the trailing slash at the end of the URL 
        // or your classes won't be found.
        //       
	String location = "file:/home/rmi_tutorial/activation/";

	// Create the rest of the parameters that will be passed to
	// the ActivationDesc constructor
	//
	MarshalledObject data = null;

	// The second argument to the ActivationDesc constructor will be used  
	// to uniquely identify this class; it's location is relative to the  
	// URL-formatted String, location.
	//
	ActivationDesc desc = new ActivationDesc 
	    ("examples.activation.ActivatableImplementation", location, data);
	
	// Register with rmid
	//
	mri = (MyRemoteInterface)Activatable.register(desc);
	System.out.println("Got the stub for the ActivatableImplementation");
        
	// Bind the stub to a name in the registry running on 1099
	// 
	Naming.rebind("ActivatableImplementation", mri);
	System.out.println("Exported ActivatableImplementation");

	System.exit(0);
    }

}
