/*
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package examples.rmisocfac;

import java.io.*;
import java.net.*;
import java.rmi.*;
import java.rmi.server.*;

public class MultiRMISocketFactory extends RMISocketFactory {
   
    /* Get the default RMISocketFactory */
    private RMISocketFactory defaultFactory 
        = RMISocketFactory.getDefaultSocketFactory();

    /* 
     * Override createSocket to call the default RMISocketFactory's
     * createSocket method.
     */
    public Socket createSocket(String host, int port) throws IOException {
	return defaultFactory.createSocket(host, port);
    }

    /* 
     * Override createServerSocket to call the default
     * RMISocketFactory's createServerSocket method.
     */
    public ServerSocket createServerSocket(int port) throws IOException {
	return defaultFactory.createServerSocket(port);
    }

    /*
     * Override createSocket to create the type of socket specified by
     * the SocketType parameter.
     */
    public Socket createSocket(String host, int port, SocketType type)
	throws IOException
    {
        Socket socket;
	String protocol = type.getProtocol();
        if(protocol.equals("xor"))
	    // Use default pattern.
	    socket = new XorSocket(host, port, (byte)0);
	else if (protocol.equals("compression")) 
	    socket = new CompressionSocket(host, port);
	else 
	    throw new IOException("protocol " + protocol + " not supported");

	return socket;
    }

    /*
     * Override createServerSocket to create the type of socket
     * specified by the SocketType parameter.
     */
    public ServerSocket createServerSocket(int port, SocketType type)
	throws IOException
    {
        ServerSocket server;
	String protocol = type.getProtocol();
        if(protocol.equals("xor")) 
	    // Use default pattern.
	    server = new XorServerSocket(port, (byte)0);
	else if (protocol.equals("compression")) 
            server = new CompressionServerSocket(port);
	else 
	    throw new IOException("protocol " + protocol + " not supported");

	return server;
    }
}
