/*
 * @(#)HtmlWriter.java	1.20 98/04/30
 *
 * Copyright 1997, 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package com.sun.tools.doclets;

import com.sun.javadoc.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/**
* Class for the Html Format Code Generation. 
* Initilizes PrintWriter with FileWriter, so that to use all the print
* related methods to generate the code to the named File through FileWriter.
*
* @since JDK1.2
* @author Atul M Dambalkar 
*/


public class HtmlWriter extends PrintWriter {

    protected final String htmlFilename;

    public static final String fileseparator = DirectoryManager.fileseparator;

    /**
     * Constructor, initializes PrintWriter with the FileWriter.
     *
     * @param filename File Name to which the PrintWriter will do the Output.
     * @exception IOException Exception raised by the FileWriter is passed on
     * to next level.
     */
    public HtmlWriter(String filename, String docencoding) 
                      throws IOException, UnsupportedEncodingException {
        super(genWriter(null, filename, docencoding));
        htmlFilename = filename;
    }

    public HtmlWriter(String path, String filename, String docencoding) 
                      throws IOException, UnsupportedEncodingException {
        super(genWriter(path, filename, docencoding));
        htmlFilename = filename;
    }

    static Writer genWriter(String path, String filename, String docencoding) 
                            throws IOException, UnsupportedEncodingException {
        FileOutputStream fos;
        if (path != null) {
            DirectoryManager.createDirectory(path);
            fos = new FileOutputStream(((path.length() > 0)? 
                                          path + fileseparator: "") + filename);
        } else {
            fos = new FileOutputStream(filename);
        }
        if (docencoding == null) {
            return new OutputStreamWriter(fos);
        } else {
            return new OutputStreamWriter(fos, docencoding);
        }
    }

    public void html() {
        println("<html>");
    }

    public void htmlEnd() {
        println("</html>");
    }

    public void body() {
        println("<body>");
    }
    
    public void body(String bgcolor) {
        println("<body bgcolor=\"" + bgcolor + "\">");
    }

    public void body(String bgcolor, String onload) {
        println("<body bgcolor=\"" + bgcolor + "\" onLoad=\"" + onload + "\">");
    }

    public void bodyEnd() {
        println("</body>");
    }

    public void title() {
        println("<title>");
    }
    
    public void titleEnd() {
        println("</title>");
    }

    public void ul() {
        println("<ul>");
    }

    public void ulEnd() {
        println("</ul>");
    }

    public void li() {
        print("<li>");
    }

    public void li(String type) {
        print("<li type=\"" + type + "\">");
    }

    public void h1() {
        println("<h1>");
    }

    public void h1End() {
        println("</h1>");
    }

    public void h1(String text) {
        h1();
        println(text);
        h1End();
    }

    public void h2() {
        println("<h2>");
    }

    public void h2(String text) {
        h2();
        println(text);
        h2End();
    }

    public void h2End() {
        println("</h2>");
    }

    public void h3() {
        println("<h3>");
    }

    public void h3(String text) {
        h3();
        println(text);
        h3End();
    }

    public void h3End() {
        println("</h3>");
    }

    public void h4() {
        println("<h4>");
    }

    public void h4End() {
        println("</h4>");
    }

    public void h4(String text) {
        h4();
        println(text);
        h4End();
    }

    public void h5() {
        println("<h5>");
    }

    public void h5End() {
        println("</h5>");
    }

    public void img(String imggif, String imgname, int width, int height) {
        println("<img src=\"images/" + imggif + ".gif\"" 
              + " width=\"" + width + "\" height=\"" + height 
              + "\" alt=\"" + imgname + "\">");
    }

    public void menu() {
        println("<menu>");
    }

    public void menuEnd() {
        println("</menu>");
    }

    public void pre() {
        println("<pre>");
    }

    public void preEnd() {
        println("</pre>");
    }
    
    public void hr() {
        println("<hr>");
    }

    public void hr(int size, int widthPercent) {
        println("<hr size=\"" + size + "\" width=\"" + widthPercent + "%\">");
    }

    public void hr(int size, String noshade) {
        println("<hr size=\"" + size + "\" noshade>");
    }

    public void bold() {
        print("<b>");
    }

    public void boldEnd() {
        print("</b>");
    }

    public void bold(String text) {
        bold();
        print(text);
        boldEnd();
    }
    
    public void italics(String text) {
        print("<i>");
        print(text);
        println("</i>");
    }

    public String italicsText(String text) {
        return "<i>" + text + "</i>";
    }

    public void space() {
        print("&nbsp;");
    }
        
    public void dl() {
        println("<dl>");
    }

    public void dlEnd() {
        println("</dl>");
    }
    
    public void dt() {
        print("<dt>");
    }

    public void dd() {
        print("<dd>");
    }

    public void ddEnd() {
        println("</dd>");
    }

    public void sup() {
        println("<sup>");
    }

    public void supEnd() {
        println("</sup>");
    }
    
    public void font(String size) {
        println("<font size=\"" + size + "\">");
    }

    public void fontClass(String classname) {
        println("<font class=\"" + classname + "\">");
    }
    
    public void fontSizeClass(String size, String classname) {
        println("<font size=\"" + size + "\" class=\"" + classname + "\">");
    }
    
    public void fontEnd() {
        println("</font>");
    }
   
    public void fontColor(String color) {
        println("<font color=\"" + color + "\">");
    }
 
    public void center() {
        println("<center>");
    }

    public void centerEnd() {
        println("</center>");
    }
    
    public void aName(String name) {
        print("<a name=\"" + name + "\">");
    }

    public void aEnd() {
        print("</a>");
    }

    public void italic() {
        print("<i>");
    }

    public void italicEnd() {
        print("</i>");
    }

    public void anchor(String name, String content) {
        aName(name);
        print(content);
        aEnd();
    }

    public void anchor(String name) {
        aName(name);
        print("<!-- -->");
        aEnd();
    }

    public void p() {
        println();
        println("<p>");
    }

    public void br() {
        println();
        println("<br>");
    }

    public void address() {
        println("<address>");
    }

    public void addressEnd() {
        println("</address>");
    }
    
    public void head() {
        println("<head>");
    }

    public void headEnd() {
        println("</head>");
    }
    
    public void code() {
        println("<code>");
    }

    public void codeEnd() {
        println("</code>");
    }
    
    public void em() {
        println("<em>");
    }

    public void emEnd() {
        println("</em>");
    }
    
    public void table(int border, String width) {
        println("\n<table BORDER=\"" + border 
                + "\" WIDTH=\"" + width + "\">");
    } 
    
    public void table() {
        table(0, "100%"); 
    } 
    
    public void tableEnd() {
        println("</table>");
    }

    public void tr() {
        println("<tr>");
    }

    public void trEnd() {
        println("</tr>");
    }
   
    public void td() {
        print("<td>");
    }

    public void tdNowrap() {
        print("<td nowrap>");
    }

    public void tdWidth(String width) {
        print("<td width=\"" + width + "\">");
    }

    public void tdEnd() {
        println("</td>");
    }

    public void link(String str) {
        println("<link " + str + ">");
    }

    public void commentStart() {
         print("<!-- ");
    }
 
    public void commentEnd() {
         println("-->");
    }
 
    public void trBgcolorClass(String color, String classname) {
        println("<tr BGCOLOR=\"" + color + "\" class=\"" + classname + "\">");
    }

    public void trBgcolor(String color) {
        println("<tr BGCOLOR=\"" + color + "\">");
    }
    public void tdColspan(int i) {
        print("<td colspan=" + i + ">");
    }
        
    public void tdAlign(String str) {
        print("<td align=" + str + ">");
    }

    public void tdVAlign(String str) {
        print("<td valign=" + str + ">");
    }

    public void tdAlignRowspan(String str, int span) {
        print("<td align=" + str + " rowspan=" + span + ">");
    }

    public void tdAlignVAlignRowspan(String align, String valign, int span) {
        print("<td align=" + align + " valign=" + valign 
                + " rowspan=" + span + ">");
    }

    public void blockquote() {
        println("<blockquote>");
    }

    public void blockquoteEnd() {
        println("</blockquote>");
    }
}
