/*
 * @(#)HelpWriter.java	1.1 98/04/21
 *
 * Copyright 1997, 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package com.sun.tools.doclets.standard;

import com.sun.tools.doclets.*;
import com.sun.tools.javadoc.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/**
 * Generate the Help File for the generated documentation.
 *
 * @author Atul M Dambalkar 
 */
public class HelpWriter extends HtmlStandardWriter {

    /**
     * Constructor.
     */
    public HelpWriter(String filename) throws IOException {
        super(filename);
    }

    /**
     * Generate the help file.
     */
    public static void generate() throws DocletAbortException {
        HelpWriter helpgen;
        String filename = "";
        try {
            if (Standard.configuration().helpfile.length() == 0 && 
                                          !Standard.configuration().nohelp) {
                filename = "help.html";
                helpgen = new HelpWriter(filename);
                helpgen.generateHelpFile();
                helpgen.close();
            }
        } catch (IOException exc) {
            Standard.configuration().standardmessage.error(
                "doclet.exception_encountered",
                exc.toString(), filename);
            throw new DocletAbortException();
        }
    }

    /**
     * Generate the help file.
     */
    protected void generateHelpFile() {
        printHeader(getText("doclet.Help_title")); 
        navLinks(true);  hr();

        printHelpFileContents();

        navLinks(false);
        printFooter();
    }

    /**
     * Print the help file contents.
     */
    protected void printHelpFileContents() {
        center(); h1(); printText("doclet.Help_line_1"); h1End(); centerEnd();
        printText("doclet.Help_line_2");
        h3(); printText("doclet.Overview"); h3End();
        blockquote(); p(); printText("doclet.Help_line_3"); blockquoteEnd();
        h3(); printText("doclet.Package"); h3End();
        blockquote(); p(); printText("doclet.Help_line_4"); 
        ul(); 
        li(); printText("doclet.Interfaces");
        li(); printText("doclet.Classes");
        li(); printText("doclet.Exceptions");
        li(); printText("doclet.Errors");
        ulEnd(); 
        blockquoteEnd();
        h3(); printText("doclet.Help_line_5"); h3End();
        blockquote(); p(); printText("doclet.Help_line_6"); 
        ul();
        li(); printText("doclet.Help_line_7");
        li(); printText("doclet.Help_line_8");
        li(); printText("doclet.Help_line_9");
        li(); printText("doclet.Help_line_10");
        li(); printText("doclet.Help_line_11");
        p();
        li(); printText("doclet.Inner_Class_Summary");
        li(); printText("doclet.Field_Summary");
        li(); printText("doclet.Constructor_Summary");
        li(); printText("doclet.Method_Summary");
        p();
        li(); printText("doclet.Field_Detail");
        li(); printText("doclet.Constructor_Detail");
        li(); printText("doclet.Method_Detail");
        ulEnd(); 
        printText("doclet.Help_line_12");
        blockquoteEnd();
        h3(); printText("doclet.Help_line_13"); h3End();
        blockquote(); 
        printText("doclet.The"); print(' '); 
        printHyperLink("tree.html", getText("doclet.Class_Hierarchy")); 
        print(' ');
        printText("doclet.Help_line_14_without_tree_link");
        ul(); 
        li(); printText("doclet.Help_line_15");
        li(); printText("doclet.Help_line_16");
        ulEnd(); 
        blockquoteEnd();
        h3(); printText("doclet.Deprecated_API"); h3End();
        blockquote();
        printText("doclet.The"); print(' '); 
        printHyperLink("deprecatedlist.html", 
                                          getText("doclet.Deprecated_API")); 
        print(' ');
        printText("doclet.Help_line_17_without_deprecated_api_link");
        blockquoteEnd();
        blockquote();
        printText("doclet.Help_line_18");
        blockquoteEnd();
        hr();
        font("-1"); printText("doclet.Help_line_19"); fontEnd();
    }

    protected void navLinkHelp() {
        boldText("doclet.Help");
    }
}



