/*
 * @(#)PackageIndexFrameWriter.java	1.7 98/04/21
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package com.sun.tools.doclets.standard;

import com.sun.tools.doclets.*;
import com.sun.javadoc.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/**
 * Write out the package index.
 *
 * @see com.sun.javadoc.PackageDoc
 * @see com.sun.tools.doclets.HtmlDocWriter
 * @author Atul M Dambalkar 
 */
public class PackageIndexFrameWriter extends AbstractPackageIndexWriter {

    /**
     * Constructor.
     */
    public PackageIndexFrameWriter(String filename, 
                                   RootDoc root) throws IOException {
        super(filename, root);
    }

    /**
     * Generate the package index.
     *
     * @param root the root of the doc tree.
     */
    public static void generate(RootDoc root) throws DocletAbortException {
        PackageIndexFrameWriter packgen;
        String filename = "overview-frame.html";
        try {
            packgen = new PackageIndexFrameWriter(filename, root);
            packgen.generatePackageIndexFile();
            packgen.close();
        } catch (IOException exc) {
 Standard.configuration().standardmessage.error("doclet.exception_encountered", 
                                                 exc.toString(), filename);
            throw new DocletAbortException();
        }
    }

    protected void printIndexRow(PackageDoc packagedoc) {
        String dirname = DirectoryManager.createPathString(packagedoc);
        fontClass("FrameItemFont");
        printTargetHyperLink(dirname + fileseparator + "package-frame.html", 
                             "packageFrame", packagedoc.name());
        fontEnd();
        br();
    }

    protected void printNavigationBarHeader() {
        if (Standard.configuration().header != null) {
            printTableHeader();
            fontSizeClass("+1", "FrameTitleFont");
            bold(Standard.configuration().header);
            fontEnd();
            printTableFooter();
        }
    }

    protected void printIndexHeader(String textKey) {
        printTableHeader();
    }
   
    protected void printIndexFooter() {
        printTableFooter();
    }

    protected void printAllClassesPackagesLink() {
        fontClass("FrameItemFont");
        printTargetHyperLink("allclasses-frame.html", "packageFrame", 
                              getText("doclet.All_Classes"));
        fontEnd();
        p();
        fontSizeClass("+1", "FrameHeadingFont");
        printText("doclet.Packages");
        fontEnd();
        br();
    }

    protected void printNavigationBarFooter() {
        p();
        space();
    }

    protected void printTableFooter() {
        tdEnd();
        trEnd();
        tableEnd();
    }

    protected void printTableHeader() {
        table();
        tr();
        tdNowrap();
    }
}



