/*
 * Copyright (c) 1997 John Jensen. All rights reserved.
 *
 * This software is FREE FOR COMMERCIAL AND NON-COMMERCIAL USE,
 * provided the following condition is met.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that any copy or derivative of this software or documentation
 * retaining the name "John Jensen" also retains this condition and the
 * following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * CopyrightVersion 1.0
 */

import java.io.*;
import java.awt.*;
import java.util.*;

public class StringMan
{
	Properties strings = null;

	private String prop_name = new String("strings.dat");

	private void setDefaults()
	{
		strings = new Properties();

		// It's a pain to repeat identical strings, but that's what
		// you're supposed to do.  The strings may translate differently,
		// something that might work as a noun and a verb in English might
		// not in German, etc.

		strings.put("MenuFile","File");
		strings.put("MenuNew","New");
		strings.put("MenuOpen","Open");
		strings.put("MenuSave","Save");
		strings.put("MenuSaveAs","Save As");
		strings.put("MenuPrint","Print");
		strings.put("MenuClose","Close");

		strings.put("MenuEdit","Edit");
		strings.put("MenuUndo","Undo");
		strings.put("MenuRedo","Redo");
		strings.put("MenuCopy","Copy");
		strings.put("MenuCut","Cut");
		strings.put("MenuPaste","Paste");
		strings.put("MenuFind","Find");
		strings.put("MenuReplace","Replace");

		strings.put("MenuWindow","Window");
		strings.put("MenuCloneWin","Clone Window");

		strings.put("MenuHelp","Help");
		strings.put("MenuAbout","About");

		strings.put("DialogOpen","Open text file");
		strings.put("DialogSaveAs","Save file as");
		strings.put("DialogFind","Find");
		strings.put("DialogReplace","Find and Replace");
		strings.put("DialogNotFound","String Not Found");
		strings.put("DialogAbout","About mpEDIT");
		strings.put("DialogDirty","Warning!");

		strings.put("PromptFind","Find:");
		strings.put("PromptReplace","Replace:");

		strings.put("ButtonOk","OK");
		strings.put("ButtonCancel","Cancel");
		strings.put("ButtonClose","Close");
		strings.put("ButtonFindNext","Find Next");
		strings.put("ButtonReplace","Replace");
		strings.put("ButtonSave","Save");
		strings.put("ButtonDontSave","Don't Save");

		strings.put("UntitledDoc","Untitled");

		strings.put("TextAbout1","mpEDIT v1.01");
		strings.put("TextAbout2","Copyright (c) 1997 John Jensen");
		strings.put("TextAbout3","Contact: jjens@primenet.com");
		strings.put("TextAbout4","This is experimental software, see");
		strings.put("TextAbout5","http://members.tripod.com/~mpTOOLS");
		strings.put("TextAbout6","for details.");

		strings.put("TextDirty","This file has not been saved.");
	}

	public Properties getStrings()
	{
		if (strings == null)
		{
			strings = new Properties();
			setDefaults();

			try
			{
				FileInputStream sf = new FileInputStream(prop_name);
				strings.load( sf );
				sf.close();
			}

			catch (Exception e)
			{
				System.out.println("Warning: strings could not be read, using defaults");
			}
		}

		return strings;
	}

	public void writeStrings()
	{
		try
		{
			FileOutputStream od = new FileOutputStream(prop_name);
			strings.save( od, "Strings for mpEdit" );
			od.close();
		}

		catch (Exception e)
		{
			System.out.println("Warning: strings could not be saved");
		}
	}
}