/*
 * Copyright (c) 1997 John Jensen. All rights reserved.
 *
 * This software is FREE FOR COMMERCIAL AND NON-COMMERCIAL USE,
 * provided the following condition is met.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that any copy or derivative of this software or documentation
 * retaining the name "John Jensen" also retains this condition and the
 * following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * CopyrightVersion 1.0
 */

import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;

public class TextFrame extends Frame implements WindowListener
{
	// the parent

	mpEDIT		mpEdit;
	Properties	strings;
	PropMan		propMan;
	DocMan		docMan;

	// these three .. no four! .. four objects form the main window

	private Scrollbar	horiz;
	private Scrollbar	vert;
	private TextMenu	textMenu;
	private TextCanvas	textCanvas;

	public TextFrame(mpEDIT mpe, Properties str, PropMan pm, DocMan doc)
	{
		super.setTitle("mpEDIT");

		mpEdit = mpe;
		strings = str;
		propMan = pm;
		docMan = doc;

		vert = new Scrollbar(Scrollbar.VERTICAL);
		add("East", vert);

		horiz = new Scrollbar(Scrollbar.HORIZONTAL);
		add("South", horiz);

		textCanvas = new TextCanvas(propMan,doc,this,horiz,vert);
		add("Center", textCanvas);

		textMenu = new TextMenu(mpEdit,strings,propMan,docMan,this);
		setMenuBar(textMenu);

		addWindowListener(this);
	}

	public TextCanvas getCanvas()
	{
		return textCanvas;
	}

	public Point getPlace(Dimension size)
	{
		return mpEdit.getPlace(size);
	}
	
	public void clearCanvas()
	{
		 textCanvas.clear();
	}
	
	public void redoCanvas()
	{
		 textCanvas.redoCanvas();
		 textCanvas.repaint();
//		 textCanvas.requestFocus();
	}

	public void legalizeCursor()
	{
		 textCanvas.legalizeCursor();
	}

	public void release_cursor()
	{
		 textCanvas.release_cursor();
	}

	public void pause_cursor()
	{
		 textCanvas.pause_cursor();
	}
		
	public void setPos(TextPosition tp)
	{
		 textCanvas.setPos(tp);
	}
		
	// add current filename to window title

	public void setTitle(String name)
	{
		super.setTitle("mpEDIT - " + name);
	}
	
	// add the 1.1 WindowListener stuff

    public void windowDeiconified(WindowEvent event) {}
    public void windowIconified(WindowEvent event) {}
    public void windowActivated(WindowEvent event) {}
    public void windowDeactivated(WindowEvent event) {}
    public void windowOpened(WindowEvent event) {}
    public void windowClosed(WindowEvent event) {}

    public void windowClosing(WindowEvent event) {
		// unsafe exit, a real editor would check for changed files
		docMan.closeFrame(this);
    }

}

