/*
 * Copyright (c) 1997 John Jensen. All rights reserved.
 *
 * This software is FREE FOR COMMERCIAL AND NON-COMMERCIAL USE,
 * provided the following condition is met.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that any copy or derivative of this software or documentation
 * retaining the name "John Jensen" also retains this condition and the
 * following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * CopyrightVersion 1.0
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;

// TextMenu is a fairly large class which creates and first handles
// events from all menus.  This may all change when I add toobars.

public class TextMenu extends MenuBar implements ActionListener, ClipboardOwner
{
	// remember who our friends are

	private mpEDIT mpEdit;
	private TextFrame textFrame;
	private TextCanvas textCanvas;
	private PropMan propMan;
	private DocMan docMan;
	private Properties strings;

	// retain standard strings

	private String fileString;
	private String newString;
	private String openString;
	private String saveString;
	private String saveAsString;
	private String printString;
	private String closeString;
	private String editString;
	private String undoString;
	private String redoString;
	private String windowString;
	private String cloneWinString;
	private String helpString;
	private String aboutString;
	private String copyString;
	private String cutString;
	private String pasteString;
	private String findString;
	private String replaceString;

	// internal data

	private Menu fileMenu;
	private Menu editMenu;
	private Menu windowMenu;
	private Menu helpMenu;

	private	MenuItem undoItem;
	private	MenuItem redoItem;
	private	MenuItem copyItem;
	private	MenuItem cutItem;

	private Clipboard clipboard;

	// constructor puts together UI

	public TextMenu(mpEDIT mpe, Properties str, PropMan pm, DocMan doc, TextFrame tf)
	{		
		super();

		mpEdit = mpe;
		strings = str;
		propMan = pm;
		docMan = doc;
		textFrame = tf;
		textCanvas = textFrame.getCanvas();

		clipboard = textFrame.getToolkit().getSystemClipboard();

		loadStrings();

		makeFileMenu();
		makeEditMenu();
		makeWindowMenu();
		makeHelpMenu();

		docMan.addTextMenu(this);
		textCanvas.setTextMenu(this);
	}

	private void makeFileMenu()
	{
		MenuItem item;
		fileMenu = new Menu(fileString);
		fileMenu.add(new MenuItem(newString));
		fileMenu.add(new MenuItem(openString));
		fileMenu.add(new MenuItem(saveString));
		fileMenu.add(new MenuItem(saveAsString));
		fileMenu.addSeparator();
		item = new MenuItem(printString);
		fileMenu.add(item);
		item.setEnabled(false);
		fileMenu.addSeparator();
		fileMenu.add(new MenuItem(closeString));    
		fileMenu.addActionListener(this);        
		add(fileMenu);
	}

	private void makeEditMenu()
	{
		MenuItem item;

		editMenu = new Menu(editString);
		undoItem = new MenuItem(undoString, new MenuShortcut(KeyEvent.VK_Z,false));
		undoItem.setActionCommand(undoString);
		undoItem.setEnabled(false);
		editMenu.add(undoItem);
		redoItem = new MenuItem(redoString, new MenuShortcut(KeyEvent.VK_Y,false));
		redoItem.setActionCommand(redoString);
		redoItem.setEnabled(false);
		editMenu.add(redoItem);
		editMenu.addSeparator();
		copyItem = new MenuItem(copyString, new MenuShortcut(KeyEvent.VK_C,false));
		copyItem.setActionCommand(copyString);
		copyItem.setEnabled(false);
		editMenu.add(copyItem);
		cutItem = new MenuItem(cutString, new MenuShortcut(KeyEvent.VK_X,false));
		cutItem.setActionCommand(cutString);
		cutItem.setEnabled(false);
		editMenu.add(cutItem);
		item = new MenuItem(pasteString, new MenuShortcut(KeyEvent.VK_V,false));
		item.setActionCommand(pasteString);
		editMenu.add(item);
		editMenu.addSeparator();
		item = new MenuItem(findString, new MenuShortcut(KeyEvent.VK_F,false));
		item.setActionCommand(findString);
		editMenu.add(item);
		item = new MenuItem(replaceString, new MenuShortcut(KeyEvent.VK_H,false));
		item.setActionCommand(replaceString);
		editMenu.add(item);
		editMenu.addActionListener(this);   
		add(editMenu);
	}
	
	private void makeWindowMenu()
	{
		windowMenu = new Menu(windowString);
		windowMenu.add(new MenuItem(cloneWinString));
		windowMenu.addActionListener(this);        
		add(windowMenu);
	}

	private void makeHelpMenu()
	{
		helpMenu = new Menu(helpString);
		helpMenu.add(new MenuItem(aboutString));
		helpMenu.addActionListener(this);        
		add(helpMenu);
	}

	private void loadStrings()
	{
		// file menu

		fileString		= strings.getProperty("MenuFile");
		newString		= strings.getProperty("MenuNew");
		openString		= strings.getProperty("MenuOpen");
		saveString		= strings.getProperty("MenuSave");
		saveAsString	= strings.getProperty("MenuSaveAs");
		printString		= strings.getProperty("MenuPrint");
		closeString		= strings.getProperty("MenuClose");

		// edit menu

		editString		= strings.getProperty("MenuEdit");
		undoString		= strings.getProperty("MenuUndo");
		redoString		= strings.getProperty("MenuRedo");
		copyString		= strings.getProperty("MenuCopy");
		cutString		= strings.getProperty("MenuCut");
		pasteString		= strings.getProperty("MenuPaste");
		findString		= strings.getProperty("MenuFind");
		replaceString	= strings.getProperty("MenuReplace");

		// help menu

		windowString	= strings.getProperty("MenuWindow");
		cloneWinString	= strings.getProperty("MenuCloneWin");

		// help menu

		helpString		= strings.getProperty("MenuHelp");
		aboutString		= strings.getProperty("MenuAbout");
	}

	// update menus in response to external events

	public void updateCopyItems(boolean have_selection)
	{
		copyItem.setEnabled(have_selection);
		cutItem.setEnabled(have_selection);
	}

	public void updateUndoItems(boolean have_undo,boolean have_redo)
	{
		undoItem.setEnabled(have_undo);
		redoItem.setEnabled(have_redo);
	}

	// action handler, part of std java event handling

    public void actionPerformed(ActionEvent evt)
	{
		int i;

		Object source = evt.getSource();
		String cmd = evt.getActionCommand();

		if (cmd == null)
			return;

		if (source == fileMenu)
			doFile(cmd);
		else
		if (source == editMenu)
			doEdit(cmd);
		else
		if (source == windowMenu)
			doWindow(cmd);
		else
		if (source == helpMenu)
		{
			AboutDialog ab = new AboutDialog(textFrame,strings,strings.getProperty("DialogAbout"));
			ab.show();
		}
	}
	
	// file menu actions

	private void doFile(String cmd)
	{
		if (cmd.equals(newString))
			mpEdit.newDoc();
		else
		if (cmd.equals(openString))
			mpEdit.openDoc(docMan,textFrame);
		else
		if (cmd.equals(saveString))
			docMan.fileSave(textFrame);
		else
		if (cmd.equals(saveAsString))
			docMan.fileSaveAs(textFrame);
		else
/*
		if (cmd.equals(printString))
			docMan.cmdPrint();
		else
*/
		if (cmd.equals(closeString))
		{
			docMan.closeFrame(textFrame);
		}
	}

	private void doEdit(String cmd)
	{
		if (cmd.equals(undoString))
		{
			docMan.undo(textFrame);
		}
		if (cmd.equals(redoString))
		{
			docMan.redo(textFrame);
		}
		else
        if (cmd.equals(copyString))
		{
			// Implement Copy operation
			String srcData = textCanvas.copy(false,false);
			if (srcData != null)
			{
                StringSelection contents = new StringSelection(srcData);
                clipboard.setContents(contents, this);
			}
        }
		else
        if (cmd.equals(cutString))
		{
			// Implement Copy operation
			String srcData = textCanvas.copy(true,true);
			if (srcData != null)
			{
                StringSelection contents = new StringSelection(srcData);
                clipboard.setContents(contents, this);
			}
        }
		else
		if (cmd.equals(pasteString))
		{
			String dstData;
            // Implement Paste operation
            Transferable content = clipboard.getContents(this);
            if (content != null)
			{
                try
				{
                    dstData = (String)content.getTransferData(DataFlavor.stringFlavor);
                }
				catch (Exception e)
				{
                    System.out.println("Could not read clipboard"); 
					return;
                }
				textCanvas.paste(dstData);
           }
        }
		else
		if (cmd.equals(findString))
		{
			FindDialog findDialog;
			findDialog = new FindDialog(textFrame,
										this,
										strings,
										strings.getProperty("DialogFind"),
										mpEdit.getFindPattern());
			findDialog.show();
		}
		else
		if (cmd.equals(replaceString))
		{
			ReplaceDialog replaceDialog;
			replaceDialog = new ReplaceDialog(textFrame,
											  this,
											  strings,
											  strings.getProperty("DialogReplace"),
											  mpEdit.getReplacePattern(),
											  mpEdit.getReplaceReplace());
			replaceDialog.show();
		}
    }

	private void doWindow(String cmd)
	{
		if (cmd.equals(cloneWinString))
		{
			Dimension size = mpEdit.getWindowSize();
			Point place = mpEdit.getPlace(size);
			TextFrame textFrame = docMan.newFrame(place,size);
			docMan.setTitles();
			textFrame.setVisible(true);
		}
	}
	
	public void CloseFindDialog(String pat)
	{
		mpEdit.setFindPattern(pat);
	}

	public void CloseReplaceDialog(String pat, String rep)
	{
		mpEdit.setReplacePattern(pat);
		mpEdit.setReplaceReplace(rep);
	}

    public void lostOwnership(Clipboard clipboard, Transferable contents)
	{
    }

}
