package origjava.test;

import java.awt.*;
import java.util.StringTokenizer;
import java.net.URL;

public class RJLMultiLineLabel extends Canvas
{
    public static final int LEFT = 0;
    public static final int CENTER = 1;
    public static final int RIGHT = 2;
    private String text;
    private String lines[];
    private int num_lines;
    private int line_height;
    private int line_ascent;
    private int line_widths[];
    private int max_width;
    private int alignment;
    private boolean border;
    private int topBottomMargin;
    private int leftRightMargin;
    private RJLFileName filename;
    private int x = 0;
    private int y = 0;
    Dimension offDimension;
    Image offImage;
    Graphics offGraphics;



    public void addNotify()
    {
        super.addNotify();
        measure();
    }

    public void setX(int i) {
        x = i;
    }
    public void setY(int i) {
        y = i;
    }

    public RJLMultiLineLabel(String string, int i, boolean flag)
    {
        setFilename(null);
        setAlignment(i);
        setText(string);
        setBorder(flag);
    }

    public RJLMultiLineLabel(String string, int i)
    {
        this(string, i, false);
    }

    public RJLMultiLineLabel(String string)
    {
        this(string, 0);
    }

    public RJLMultiLineLabel()
    {
        this(RJLRuntime.EmptyString, 0);
    }

    public RJLFileName getFilename()
    {
        return filename;
    }

    public void setFilename(RJLFileName rJLFileName)
    {
        if (rJLFileName != null)
        {
            String l_text;
            Object l_object;
            URL l_url;
            filename = rJLFileName;
            try {
                 l_url =  new URL(filename.toString());
            }
            catch (java.net.MalformedURLException ex1 )
            {
                return;
            }
            if (l_url == null ) return ;
            try {
                l_object = l_url.getContent();
            }
            catch (java.io.IOException ex2)
            {
                return;
            }
            if (l_object != null) {
                l_text = (String) l_object;
                if (l_text !=null )setText(l_text );
            }
            return;
        }
        filename = new RJLFileName();
    }


    public int getLeftRightMargin()
    {
        return leftRightMargin;
    }

    public void setLeftRightMargin(int i)
    {
        if (i >= 0)
            leftRightMargin = i;
    }

    public int getAlignment()
    {
        return alignment;
    }

    private void measure()
    {
        Font font = getFont();
        
        if (font != null)
        {   
            FontMetrics fontMetrics = getFontMetrics(font);
            if (fontMetrics != null)
            {
                line_height = fontMetrics.getHeight();
                line_ascent = fontMetrics.getAscent();
                max_width = 0;
                for (int i = 0; i < num_lines; i++)
                {
                    line_widths[i] = fontMetrics.stringWidth(lines[i]);
                    if (line_widths[i] > max_width)
                        max_width = line_widths[i];
                }
            }
        }
    }

    public void setAlignment(int i)
    {
        switch (alignment)
        {
        case 0:
        case 1:
        case 2:
            alignment = i;
            break;

        default:
            throw new IllegalArgumentException();
        }
        repaint();
        return;
    }

    public int getTopBottomMargin()
    {
        return topBottomMargin;
    }

    public void setTopBottomMargin(int i)
    {
        if (i >= 0)
            topBottomMargin = i;
    }

    public void setFont(Font font)
    {
        super.setFont(font);
        measure();
        repaint();
    }

    public Dimension minimumSize()
    {
        Dimension dimension = new Dimension(max_width + leftRightMargin * 2, num_lines * line_height + topBottomMargin * 2);
        if (dimension.width == 0)
            dimension.width = 10;
        if (dimension.height == 0)
            dimension.height = 10;
        return dimension;
    }

    public Dimension preferredSize()
    {
        return minimumSize();
    }

    public boolean getBorder()
    {
        return border;
    }

    public void setBorder(boolean flag)
    {
        border = flag;
    }

    public void setText(String string)
    {
        StringTokenizer stringTokenizer = new StringTokenizer(string, "\n");
        num_lines = stringTokenizer.countTokens();
        lines = new String[num_lines];
        line_widths = new int[num_lines];
        for (int i = 0; i < num_lines; i++)
            lines[i] = stringTokenizer.nextToken();
        measure();
        repaint();
        text = new String(string);
    }

    public String getText()
    {
        return text;
    }

    public void update(Graphics g)
    {
        //paint(g);
       Dimension dimension = getSize();
       if ( (offGraphics == null)
            || (dimension.width != offDimension.width)
            || (dimension.height != offDimension.height) 
          ) 
           {
            offDimension = dimension;
            offImage = createImage(dimension.width, dimension.height);
            offGraphics = offImage.getGraphics();
           }
     
        offGraphics.setColor(getBackground());
        offGraphics.fillRect(x, y, getSize().width - 1, getSize().height - 1);
        if (border)
        {
            offGraphics.setColor(RJLRuntime.FieldInnerTopLeftColor);
            offGraphics.drawRect(x, y, getSize().width - 1, getSize().height - 1);
        }
        int j = line_ascent + (dimension.height - num_lines * line_height) / 2;
        for (int k = 0; k < num_lines; )
        {
            int i;
            switch (alignment)
            {
            case 0:
                i = 0;
                break;

            case 2:
                i = dimension.width - line_widths[k];
                break;

            default:
                i = (dimension.width - line_widths[k]) / 2;
                break;
            }
            i += leftRightMargin;
            offGraphics.setColor(getForeground());
            offGraphics.drawString(lines[k], i + x, j + y);
            k++;
            j += line_height;
        }

        g.drawImage(offImage,0,0,this);
    }

    public void paint(Graphics g)
    {
      update(g);
    }
}
